/* $Id: SerializedIterator.java,v 1.1.1.1 1999/11/06 18:38:04 borg Exp $ */
/* Copyright  1999 George Reese, All Rights Reserved */
package com.imaginary.util;

import java.io.IOException;
import java.io.ObjectInputStream;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.NoSuchElementException;
import com.sun.java.util.collections.UnsupportedOperationException;

/**
 * Provides an <CODE>Iterator</CODE> interface into a stream of serialized
 * objects.
 * <BR>
 * Last modified $Date: 1999/11/06 18:38:04 $
 * @version $Revision: 1.1.1.1 $
 * @author George Reese (borg@imaginary.com)
 */
public class SerializedIterator implements Iterator {
    private transient ObjectInputStream input      = null;
    private           Object            nextObject = null;

    /**
     * Required to serialize this object.
     */
    public SerializedIterator() {
        super();
    }
    
    /**
     * Constructs a new <CODE>SerializedIterator</CODE> from the specified
     * input stream.
     * @param in the input stream from which objects are to be read
     */
    public SerializedIterator(ObjectInputStream in) {
        super();
        input = in;
    }

    /**
     * @return true if there is another object on the stream
     */
    public boolean hasNext() {
        if( nextObject != null ) {
            return true;
        }
        try {
            nextObject = input.readObject();
        }
        catch( ClassNotFoundException e ) {
            return false;
        }
        catch( IOException e ) {
            return false;
        }
        return true;
    }

    /**
     * @return the next object on the stream
     */
    public Object next() {
        Object ob;
        
        if( nextObject == null && !hasNext() ) {
            throw new NoSuchElementException("No more elements in stream.");
        }
        ob = nextObject;
        nextObject = null;
        return ob;
    }

    /**
     * This operation is unsupported in this implementation.
     * @throws java.lang.UnsupportedOperationException always thrown
     */
    public void remove() {
        throw new UnsupportedOperationException("You cannot delete from a " +
                                                "stream.");
    }
}
