//---------------------------------------------------------------------------||
//                                                                           ||
// XMLEnabler.java                                                           ||
//                                                                           ||
//---------------------------------------------------------------------------||
//                                                                           ||
// BUILD NOTE:                                                               ||
//                                                                           ||
// This code currently doesn't work with the latest version of XML4J.  To    ||
// fix this, add the version of XML4J.JAR that ships with the Lotus XSL      ||
// engine to the front of the classpath.                                     ||
//                                                                           ||
//---------------------------------------------------------------------------||
//                                                                           ||
// To-Dos:                                                                   ||
//                                                                           ||
// 1) Find out where I can put my properties file so that the servlet can    ||
//    load xmlEnabler.txt without qualifying the filename.                   ||
// 2) Find out where I can put the XML and XSL files so that the servlet can ||
//    find them.                                                             ||
// 3) Add a TaskGuide to make it easy to modify the properties file.  You    ||
//    should be able to change the stylesheets, define new browser types,    ||
//    and change the order in which the browsers are searched for.           ||
// 4) Write a small servlet that tells you what the user-Agent field is.     ||
// 5) Finish the javadoc work.                                               ||
// 6) Add an HTML page that provides a front-end to the server.              ||
// 7) Convert the XSL examples to use the 8/19/98 W3C syntax.                ||
//                                                                           ||
//---------------------------------------------------------------------------||
// Created 6/19/98 by Doug Tidwell.                                          ||
//---------------------------------------------------------------------------||

package com.ibm.xmlenabler;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.xml.parser.*;
import org.w3c.dom.*;
import com.lotus.xsl.*;
import com.lotus.xsl.xml4j.*;
import org.xml.sax.SAXException ;
/**
 * <p>This servlet transforms an XML document using a particular XSL
 * stylesheet.  The stylesheet used is determined by two factors: </p>
 * <ul>
 * <li>The browser type, as indicated by the "user-Agent" field in the
 * HTTP header, and
 * <li>The default stylesheet for that browser, as defined in the
 * properties file (XMLEnabler.txt) for this servlet.                   
 * </ul>
 * <p>Currently the XML document is passed in as a parameter (URL=) in
 * the URL itself.  This document is parsed, the appropriate XSL stylesheet
 * is parsed, then the two documents are combined.  The results of the
 * transformation are returned via HTTP to the caller.</p>
 * <h3>Sample Invocation:</h3>
 * <p>This URL invokes the servlet and returns a transformed XML
 * document:</p>
 * <pre>
 * http://localhost:8080/servlet/com.ibm.XMLEnabler.XMLEnabler?URL=http://localhost/testdata.xml
http://https://students.npac.syr.edu:5557/com.ibm.XMLEnabler.XMLEnabler?URL=https://students.npac.syr.edu:5557/students.xml

 * </pre>
 * <h3>Property File Format:</h3>
 * <p>The property file can include three types of parameters:</p>
 * <dl>
 * <dt>Default</dt>
 * <dd>Defines the URL of the default XSL stylesheet</dd>
 * <dt>Order</dt>
 * <dd>Defines the order of the browser types for which we should check.</dd>
 * <dt><i>search-strings</i></dt>
 * <dd>The name of this property is a search string that should be found
 * in the HTTP header, along with the URL of the XSL stylesheet that
 * should be used for any browser that matches the search string.</dd>
 * </dl>
 * <p>Here's a sample file format:</p>
 * <pre>
 * Default=http://localhost/xmLDAP01.xsl
 * Order=MSIE_4.0/Opera/Lynx/Mozilla
 * MSIE_4.0=http://localhost/xmLDAP01.xsl
 * Opera=http://localhost/xmLDAP02.xsl
 * Lynx=http://localhost/xmLDAP03.xsl
 * Mozilla=http://localhost/hlist.xsl
 * </pre>
 * <p>This file defines the default stylesheet, the order in which browser
 * types should be searched, and stylesheets for browsers whose HTTP
 * headers include the strings "<code>MSIE</code>" (Internet Explorer),
 * "<code>Opera</code>," "<code>Lynx</code>," and "<code>Mozilla</code>"
 * (Netscape).  It is significant in this example that "<code>MSIE_4.0</code>"
 * is queried before "<code>Mozilla</code>," because both Internet Explorer
 * and Netscape contain the string "<code>Mozilla</code>" in their HTTP
 * headers.</p>
 * <h3>Ways to Enhance This Code:</h3>
 * <ul>
 * <li>Integrate this with a proxy function so that the client doesn't have
 * to know the URL of the servlet.  It would be nice if the client asked
 * for a URL, with this servlet invisibly transforming the XML data
 * before it reaches the clients.</li>
 * </ul>
 *
 * @version     1.00, 16 Nov 1998
 * @author      Doug Tidwell, with thanks to Scott Boag for his XSL engine, and
 *              the Tokyo Research Lab team for their brilliant XML parser. 
 */

public class xmlenabler extends HttpServlet
{
  public static final String COPYRIGHT = "(C) Copyright IBM Corp. 1998.  All rights reserved.";

  public static final String LONG_COPYRIGHT =
                      "Licensed Materials - Property of IBM" +
                      "\nIBM XML Enabler" +
                      "\n(C) Copyright IBM Corp. 1998.  All rights reserved." +
                      "\n\nUS Government Users Restricted Rights - Use, duplication, or" +
                      "\ndisclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

  Properties prop = null;

  public void init(ServletConfig cfig)
  {
    // We get the properties for the servlet before anything else.
    try
    {
      FileInputStream propFile = new FileInputStream("xmlEnabler.txt");
      prop = new Properties(System.getProperties());
      prop.load(propFile);
      System.out.println("XMLEnabler initialized!");
    }
    catch (IOException ioe) {};
  }

  /**
   * The <code>service</code> method responds to an HTTP request.  
   * @param req HttpServletRequest that encapsulates the request to the
   * servlet 
   * @param resp HttpServletResponse that encapsulates the response from the
   * servlet
   * @throws IOException if detected when handling the request
   * @throws MalformedURLException if the requested URL is not valid
   */

  public void service(HttpServletRequest req, HttpServletResponse res)
      throws IOException, MalformedURLException
  {
    boolean anErrorOccurred = false;
    Document XMLDoc = null, XSLDoc = null, outputDoc = null;

    ProcessXSL xmlProcessorLiason = null;
    XSLProcessor p = null;
    
    // The contents we'll return are HTML.  We also create our output stream
    // here.  This is standard servlet stuff.
    res.setContentType("text/html");
    PrintWriter out = res.getWriter();

    // Code flow:
    //   1) Get the properties for the servlet (we did this in init())
    //   2) Get the parameters from the URL
    //   3) Create the XSL processor
    //   4) Open the requested URL and process it
    //   5) Figure out the right XSL stylesheet and process it
    //   6) Transform the document and return the results
    
    // Get the parameters from the URL
    Enumeration keys;
    String key;
    String requestedURL = "";

    keys = req.getParameterNames();
    while (keys.hasMoreElements())
    {
      key = (String) keys.nextElement();
      if (key.equalsIgnoreCase("URL"))
        requestedURL = req.getParameter(key);
    }

    URL sourceDoc = null;

    try
    {
      System.out.println(requestedURL);

      sourceDoc = new URL(requestedURL);

    }
    catch (MalformedURLException mue)
    {
      anErrorOccurred = true;
      System.out.println("catch ettim 1");
    }

    if (sourceDoc == null)
    {
      System.out.println("sourceDoc = null!");
      anErrorOccurred = true;
    }
    else
    {
      // Now we'll create the XSL processor
      xmlProcessorLiason = new ProcessXSL();
      p = new XSLProcessor(xmlProcessorLiason);

      // Open the XML document and parse it
      try {
      XMLDoc = p.parseXML(sourceDoc, 0);
      } catch (XSLProcessorException ex) { 
	System.out.println("XSLProcessorException 1");
      }
      // Get the user-Agent field from the HTTP header
      String browserType = req.getHeader("user-Agent");

      // Now we need to find the correct stylesheet.  These are defined
      // in the properties file.  First, we find the Order property; this
      // tells us the order in which we'll want to search for strings in
      // the HTTP header.  (Some strings, such as "Mozilla," may appear
      // in the header from more than one browser.)
      
      // We take each item from the Order property, get the string
      // associated with that item, then see if the HTTP header matches.
      // If none of the strings match, we get the Default property and
      // use its value as the URL of the default stylesheet. 

      if (prop == null)
      {
        anErrorOccurred = true;
        System.out.println("Something's screwed up!");
      }
      else
      {
        String searchList = prop.getProperty("Order");
        System.out.println("order = " + searchList);
        boolean browserNotFound = true;
        if (searchList != null)
        {
          String nextBrowserType;
          StringTokenizer st = new StringTokenizer(searchList, "/");
          System.out.println("Now parsing search list");
          while (st.hasMoreTokens() && browserNotFound)
          {
            String nextSearchString = "";
            nextBrowserType = st.nextToken();
            nextBrowserType = strChange(nextBrowserType, "_", " ");
            System.out.println("Now searching for: " + nextBrowserType);
            if (browserType.indexOf(nextBrowserType) >= 0)
            {
              nextSearchString = prop.getProperty(nextBrowserType);
              if (nextSearchString != null)
              { 
                browserNotFound = false;
		try {
		     System.out.println("nextSearchString "+nextSearchString);
                XSLDoc = p.parseXML(nextSearchString, 0);
		} catch (XSLProcessorException ex) { 
		  System.out.println("XSLProcessorException 2");
		}
              }
            }
          }
        }
        if (browserNotFound)
        {
          String defaultStylesheet = prop.getProperty("Default");
          System.out.println("Using default stylesheet " + defaultStylesheet);
          if (defaultStylesheet != null)
	    try {
            XSLDoc = p.parseXML(defaultStylesheet, 0);
	  } catch (XSLProcessorException ex) { 
	    System.out.println("XSLProcessorException 3");
	  }
        }
      }
    }

    if (anErrorOccurred)
    {
      out.println("<html><head><title>Property File Error</title></head>\n");
      out.println("<body><h1>Property File Error</h1>\n");
      out.println("<p>The XMLEnabler property file, <code>XMLEnabler.txt</code>, \n");
      out.println("could not be found or contained bad data.  Contact your \n");
      out.println("webmaster or webmistress for more information.</p></body></html>\n");
    }
    else
    {
      // Now that we've parsed the XML document and the XSL stylesheet,
      // we'll combine them to transform the original document. 
      // outputDoc = p.transformDocument(XMLDoc, XSLDoc, null, null);
      //xmlProcessorLiason.SaveXMLToFile(outputDoc, out);
       try {p.process(XMLDoc, XSLDoc, null,out); }
         catch (SAXException se) { }
      catch ( XSLProcessorException pe) { }
      // p.process(XMLDoc, XSLDoc, null,out);
      //  xmlProcessorLiason.process(outputDoc, out);
    }
  }

  private static String strChange(String string, String from, String to)
  {
    String before = "",
           after = "";
    int    index;

    index = string.indexOf(from);
    while (index >= 0)
    {
      before = string.substring(0, index);
      after = string.substring(index + from.length());
      string = before + to + after;

      index = string.indexOf(from, index + to.length());
    }

    return string;
  }
}
