/*
 * @(#)DBServlet.java	1.5 97/03/03
 * 
 * Copyright (c) 1995-1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 */

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.net.*;
import java.sql.*;

/**
 * DBServlet
 * 
 * This servlet demonstrates how to use JDBC stack
 * and access database using Jeeves servlet.  In order to run this
 * servlet, CLASSPATH, LD_LIBRARY_PATH, ODBC_INI, and other related
 * environment variables must be set prior to running the Jeeves
 * server since the name resolution for JDBC stack library happens
 * during the run time.  Part of the code was contributed by
 * Donald E. McCollam <mccollam@snow.CCIT.Arizona.EDU>
 * 
 * @author   Jongyoon Lee
 * @version  1.5, 03/03/97
 */

/**
 *DBServlet Modified
 *This servlet was modified to access the oracle Database on osprey7 using 
 * using Weblogic (www.weblogic.com) JDBC driver inorder to show the 3-tier 
 * JDBC example.
 */
public class DBServlet extends HttpServlet {

    /**
     * service method
     */
    public void service(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException
    {
               
        String username, password, query;
	Connection con;
	PrintStream out;

	// fetch the parameters      
	username = req.getParameter("username");
	password = req.getParameter("password");
	query = req.getParameter("query");

	res.setContentType("text/html");
	out = new PrintStream(res.getOutputStream());
	printPageHeader(out);

	// if no parameters, just print the form
	if (username == null || password == null ||
	    query == null) {
	    printPageFooter(out);
	    return;
	}

	String url	 = "jdbc:oracle:thin:@carver.npac.syr.edu:1521:europe";

	out.println("<hr><h3>Previous Query</h3>");
	out.println("<pre>");
	out.println("       User ID : "+username);
	out.println("      Password : "+password);
	out.println("         Query : "+query);
	out.println("</pre>");

	try {
	    // find the jdbc driver
		Class.forName("oracle.jdbc.driver.OracleDriver");

	    // get the connection to the database
	    con = DriverManager.getConnection(url, username, password);
	    out.println("<hr>");
	    out.println("<h3>JDBC Driver and Database Messages</h3>");
	    checkForWarning(con.getWarnings(), out);
	    DatabaseMetaData dma = con.getMetaData();
	    out.println("Connected to " + dma.getURL() + "<br>");
	    out.println("Driver       " + dma.getDriverName() + "<br>");
	    out.println("Version      " + dma.getDriverVersion() + "<br>");

	    // create and execute the query
	    Statement stmt = con.createStatement();
	    ResultSet rs = stmt.executeQuery(query);

	    // print out the result
	    dispResultSet(rs, out);
	    rs.close();
	    stmt.close();
	    con.close();
	    out.println("<hr>");
	}
	catch (SQLException ex) {
	    out.println("<hr>*** SQLException caught ***<p>");
	    while (ex != null) {
	      out.println("SQLState: " + ex.getSQLState() + "<br>");
	      out.println("Message:  " + ex.getMessage() + "<br>");
	      out.println("Vendor:   " + ex.getErrorCode() + "<br>");
	      ex = ex.getNextException();
	    }
	}
	catch (java.lang.Exception ex) {
	    ex.printStackTrace();
	}

	printPageFooter(out);
    }

    /**
     * returns the servlet info
     */ 
    public String getServletInfo() {
	return "This is a simple servlet that shows how to use JDBC with servlets";
    }


    /*
     * check if the database server has anything to say
     */
    private void checkForWarning(SQLWarning warn, PrintStream out)
          throws SQLException
    {
       boolean rc = false;
 
       if (warn != null) {
          out.println("<hr>*** Warning ***<p>");
          rc = true;
          while (warn != null) {
             out.println("SQLState: " + warn.getSQLState() + "<br>");
             out.println("Message:  " + warn.getMessage() + "<br>");
             out.println("Vendor:   " + warn.getErrorCode() + "<br>");
             warn = warn.getNextWarning();
          }
       }
    }


    /*
     * display the result in html table format
     */
    private void dispResultSet(ResultSet rs, PrintStream out)
       throws SQLException
    {
       int i;
 
       // metadata can supply information about the schema
       ResultSetMetaData rsmd = rs.getMetaData();
       int numCols = rsmd.getColumnCount();
       out.println("<hr>");
       out.println("<h3>Database Columns and Data</h3>");
       out.println("<table border=3>");
       out.println("<tr>");
       for (i=1; i<=numCols; i++) {
          out.println("<th>" + rsmd.getColumnLabel(i) + "</th>");
       }
       out.println("</tr>");

       // for entire data
       while (rs.next()) {
	  out.println("<tr>");

	  // for one row
          for (i=1; i<=numCols; i++) {
	     dispElement(rs, rsmd.getColumnType(i), out, i);
          }
	  out.println("</tr>");
       }
       out.println("</table>");
    }


    // print one element
    private void dispElement(ResultSet rs, int dataType,
			     PrintStream out, int col)
          throws SQLException
    {

        // ask for data depending on the datatype
	switch(dataType) {
	case Types.DATE:
	  java.sql.Date date = rs.getDate(col);
	  out.println("<th>" + date.toString() + "</th>");
	  break;
	case Types.TIME:
	  java.sql.Time time = rs.getTime(col);
	  out.println("<th>" + time.toString() + "</th>");
	  break;
	case Types.TIMESTAMP:
	  java.sql.Timestamp timestamp = rs.getTimestamp(col);
	  out.println("<th>" + timestamp.toString() + "</th>");
	  break;
	case Types.CHAR:
	case Types.VARCHAR:
	case Types.LONGVARCHAR:
	  String str = rs.getString(col);
	  out.println("<th>" + str + "</th>");
	  break;
	case Types.NUMERIC:
	case Types.DECIMAL:
	  java.math.BigDecimal numeric = rs.getBigDecimal(col, 10);
	  out.println("<th>" + numeric.toString() + "</th>");
	  break;
	case Types.BIT:
	  boolean bit = rs.getBoolean(col);
	  out.println("<th>" + new Boolean(bit) + "</th>");
	  break;
	case Types.TINYINT:
	  byte tinyint = rs.getByte(col);
	  out.println("<th>" + new Integer(tinyint) + "</th>");
	  break;
	case Types.SMALLINT:
	  short smallint = rs.getShort(col);
	  out.println("<th>" + new Integer(smallint) + "</th>");
	  break;
	case Types.INTEGER:
	  int integer = rs.getInt(col);
	  out.println("<th>" + new Integer(integer) + "</th>");
	  break;
	case Types.BIGINT:
	  long bigint = rs.getLong(col);
	  out.println("<th>" + new Long(bigint) + "</th>");
	  break;
	case Types.REAL:
	  float real = rs.getFloat(col);
	  out.println("<th>" + new Float(real) + "</th>");
	  break;
	case Types.FLOAT:
	case Types.DOUBLE:
	  double longreal = rs.getDouble(col);
	  out.println("<th>" + new Double(longreal) + "</th>");
	  break;
	case Types.BINARY:
	case Types.VARBINARY:
	case Types.LONGVARBINARY:
	  byte[] binary = rs.getBytes(col);
	  out.println("<th>" + new String(binary, 0) + "</th>");
	  break;
	}
    }


    private void printPageHeader(PrintStream out) {
	out.println("<html>");
	out.println("<head>");
	out.println("<tltle>Java Web Server Servlet Test</title>");
	out.println("</head>");
	out.println("<body>");
	out.println("<center><font size=5>" +
		    "<b>Java WebServer Database Servlet</b>" +
		    "</font></center>");
	out.println("<hr>");
	out.println("<form action=\"https://class-server.npac.syr.edu:5557/GS/DBServlet\" method=\"get\">");
	out.println("<pre>");
	out.println("       User ID : <input type=textarea name=username>");
	out.println("      Password : <input type=textarea name=password>");
	out.println("         Query : <input type=textarea name=query>");
	out.println("</pre>");
	out.println("<input type=submit>");
	out.println("</form>");
    }


    private void printPageFooter(PrintStream out) {
        out.println("</body>");
	out.println("</html>");
	out.flush();
    }
}
