
package gs.html;

import java.sql.SQLException;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

/**
 * 
 * 
 */
public class Parser {

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method does not take any record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml
  (
   String templateFile, 
   Hashtable fields, 
   ServletOutputStream out
    )
       throws IOException, ServletException, SQLException
  {
    parseHtml(templateFile, fields, out, null);
  }
    

  /**
   * Retrieves the HTML file and replaces the variables with their values.
   * This version of the method <b>requires</b> a record parser. Record
   * parsers handle &lt;record&gt; tags in the HTML templates.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml (
				String templateFile, 
				Hashtable fields, 
				ServletOutputStream out,
				RecordParsable rparser
				 )       throws IOException, ServletException, SQLException
  {
    //response.setContentType("text/html");
    //ServletOutputStream out = response.getOutputStream();
    //String filename = request.getPathTranslated();

    File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));
	
	String aline, record="";
	int pos;
	boolean recordCollect = false;
	boolean body=false;
	while ( ( aline = lines.readLine() ) != null )
	  {
	      if ( aline.indexOf("<body") >= 0  )  body=true;
	      if(!recordCollect)
	      {
		pos = aline.indexOf("<record>");
		if(pos >= 0 ) // this line needs a special treatment b/of the <record> tag.
		  {
		    if(pos>0) // there must be leading characters, so let parseLine handle those.
		      parseLine(fields, aline.substring(0, pos), out);
		    
		    aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
		    recordCollect = true;
		  }
		else // no record tag, just continue.
		  Parser.parseLine(fields, aline, out);
	      }

	    if(recordCollect)
	      {
		pos = aline.indexOf("</record>");
		if(pos >= 0) // <record> tag is closed.
		  {
		    record += aline.substring(0, pos); // code between tags are captured.
		    
		    // put the record treatment here!!!
		    if(rparser != null) {// caller method may not provide a record parser! 
		      while( rparser.handleRecord(fields) ) {
			parseLine(fields, record, out); 
		      }
		    } // else  // this line commented at March8 99
		      // put the record treatment here!!!
		      aline = aline.substring(pos+9);   // trim the closing record tag.
		    Parser.parseLine(fields, aline, out); // print the rest of the line.
		    recordCollect = false;
		  }
		else
		  record += "\n"+aline; // tag is not closed and start collecting
	      }
	    if (body){
		out.println("<font color=magenta>______________________________________________________________________________________________________________</font>"); 
		body=false;
		System.out.print("x");
	    }
	  }
      }
  }
  
  public static void parseLine
  (Hashtable fields,
   String aline,
   ServletOutputStream out
    )
       throws IOException, ServletException
  {
    String value="";
    int pos = aline.indexOf("$"); // each variable must start with '$'.
    while (pos >= 0 && aline.length()>1)
      {
	if( pos != 0 ) 
	  {
	    if( aline.charAt(pos-1) == '\\' )     // escape character found!
	      aline = aline.substring(0, pos-1) + aline.substring(pos);
	    out.print( aline.substring(0, pos) ); // replace esc+$ w/ $
	    aline = aline.substring(pos);         // and print the heading part.
	    // continue w/ the rest.
	  } 
	// first char after $ must be an identifier starter usu. alphabetic chars.
	// we leave the job to Java.
	else if( Character.isUnicodeIdentifierStart(aline.charAt(1)) ) {
	  aline = Parser.parseVar( fields, aline, out  ); 
	}
	else {
	  // if it is not a starter, then print $ and continue w/ the rest.
	    out.print( aline.substring(0, 1) );
	    aline = aline.substring(1);
	  }
	pos = aline.indexOf("$");
      }
    out.println( aline );
  }
  
  public static String parseVar(Hashtable fields, String aline, ServletOutputStream out)
        throws IOException, ServletException
  {
    String E="empty";
    String F="full";
    boolean zerotest=false;
    if(aline.length() > 1) 
      {
	int pos=1;  
	String word="";
	// before coming here, we made sure that there is at least one char
	// which can make us an identifier.
	boolean status=false;
	while( Character.isUnicodeIdentifierPart( aline.charAt(pos) ) ) 
	  {
	    word += aline.charAt(pos);
	    pos++;
	  }
	if (aline.charAt(pos)=='#')  {
	  status=true;  //default E F 
	  pos++;
	  if (aline.charAt(pos)=='0') {
	    pos++; 
	    zerotest=true; 
	  }
	}
	else if (aline.charAt(pos)=='*') {
	  status=true;  E=""; F="*";
	  pos++; 
	}
	else if (aline.charAt(pos)=='!') {
	  status=true;    String n="";  pos++;
	  while ( aline.charAt(pos)!='!') {
	      n+=aline.charAt(pos);
	      pos++;
	  } 
	  E=""; F=n;
	  pos++; 
	}
	else if (aline.charAt(pos)=='%') {
	  word=(String)fields.get("start")+word;
	  status=true;  
	  pos++; 
	  if (aline.charAt(pos)=='#')  
	    pos++;
	  else  {
	    E=""; F=(String)fields.get( word);
	  }
	}	
	if (aline.charAt(pos)=='`') {
	  status=true; 
	  pos++;
	  if (aline.charAt(pos)=='x') {
	    String n="";  pos++;
	    while ( aline.charAt(pos)!='`') {
	      n+=aline.charAt(pos);
	      pos++;
	    } 
	    pos++;
	    String c=(String)fields.get( word);
 	    if (c!=null)
	      F=Integer.toString(Integer.parseInt(n)*Integer.parseInt(c));
	    else E="0";
	    
	  } 
	}
	if( !word.equals("") )
	  {
	    String s=(String)fields.get(word);
	    if (status){
	      if ( s==null || s.equals("") )
		out.print(E); //prints value
	      else if (zerotest && s.equals("0") ) out.print(E);
	      else out.print(F);
	    } 
	    else {
	      if ( s==null ) s="null";
	      out.print(s);
	      }
	    aline = aline.substring(pos);
	  }
      }
    return aline;  
  }
  ///////////////////////////////==============
  public static BufferedReader parseHtmlOpen (String templateFile)      
       throws IOException, ServletException, SQLException
  {
   File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	 BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));
    return lines;
     }
 
  }
  public static String parseHtmlUntilRecord (
				Hashtable fields, 
				ServletOutputStream out,
				BufferedReader lines
				 )       throws IOException, ServletException, SQLException
  {
	String aline, record="";
	int pos;
	boolean recordCollect = false;
	while ( ( aline = lines.readLine() ) != null )  {
	  if(!recordCollect)	    {
	    pos = aline.indexOf("<record>");
	    if(pos >= 0 ) { // this line needs a special treatment b/of the <record> tag.
	      if(pos>0) // there must be leading characters, so let parseLine handle those.
		parseLine(fields, aline.substring(0, pos), out);
	      aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
	      recordCollect = true;
	    } else // no record tag, just continue.
	      Parser.parseLine(fields, aline, out);
	  }
	  else if(recordCollect)   {
	    pos = aline.indexOf("</record>");  //</record should be at the end
	    if(pos >= 0)  {// <record> tag is closed.
	      record += aline.substring(0, pos); // code between tags are captured.
	      return record;
	    } else
	      record += "\n"+aline; // tag is not closed and start collecting
	  }
	}
    return "";
  }

   ///////////////////////////////==============
 
  public static void parseHtmlAfterRecord (
				
				Hashtable fields, 
				ServletOutputStream out,
				BufferedReader lines
				  )       throws IOException, ServletException, SQLException
  {

    String aline="";
    while ( ( aline = lines.readLine() ) != null )
      {
	
	Parser.parseLine(fields, aline, out);
	
      }
  }
 
}



