/**
 * $Id$
 * 
 * Copyright (c) 1998-1999 Lotus Corporation, Inc. All Rights Reserved.
 *				This software is provided without a warranty of any kind.
 * 
 * $State$
 * 
 * @author Dan Erikson <erikson@midsouth.rr.com>
 */
package com.lotus.xsl;

import java.util.Stack;
import java.io.Writer;
import java.io.IOException;
import java.util.Hashtable;
import org.xml.sax.*;

/**
 * FormatterToHTML formats SAX-style events into XML.
 */
public class FormatterToHTML extends FormatterToXML 
{  
  static   Hashtable  s_empties;
  static   Hashtable  s_attrempties;
  
  static   String[]  s_HTMLlat1 =    
  {
    "nbsp",    "iexcl",    "cent",    "pound",    
    "curren",    "yen",    "brvbar",    "sect",    
    "uml",    "copy",    "ordf",    "laquo",    
    "not",    "shy",    "reg",    "macr",    "deg",    
    "plusmn",    "sup2",    "sup3",    "acute",    
    "micro",    "para",    "middot",    "cedil",    
    "sup1",    "ordm",    "raquo",    "frac14",    
    "frac12",    "frac34",    "iquest",    
    "Agrave",    "Aacute",    "Acirc",    
    "Atilde",    "Auml",    "Aring",    "AElig",    
    "Ccedil",    "Egrave",    "Eacute",    "Ecirc",    
    "Euml",    "Igrave",    "Iacute",    "Icirc",    
    "Iuml",    "ETH",    "Ntilde",    "Ograve",    
    "Oacute",    "Ocirc",    "Otilde",    "Ouml",    
    "times",    "Oslash",    "Ugrave",    "Uacute",    
    "Ucirc",    "Uuml",    "Yacute",    "THORN",    
    "szlig",    "agrave",    "aacute",    "acirc",    
    "atilde",    "auml",    "aring",    "aelig",    
    "ccedil",    "egrave",    "eacute",    "ecirc",    
    "euml",    "igrave",    "iacute",    "icirc",    
    "iuml",    "eth",    "ntilde",    "ograve",    
    "oacute",    "ocirc",    "otilde",    "ouml",    
    "divide",    "oslash",    "ugrave",    "uacute",    
    "ucirc",    "uuml",    "yacute",    "thorn",    
    "yuml"};

  static   String[]  HTMLsymbol1 =    
  {
    "Alpha",    "Beta",    
    "Gamma",    "Delta",    "Epsilon",    "Zeta",    
    "Eta",    "Theta",    "Iota",    "Kappa",    
    "Lambda",    "Mu",    "Nu",    "Xi",    
    "Omicron",    "Pi",    "Rho",    "Sigma",    
    "Tau",    "Upsilon",    "Phi",    "Chi",    
    "Psi",    "Omega",    "alpha",    "beta",    
    "gamma",    "delta",    "epsilon",    "zeta",    
    "eta",    "theta",    "iota",    "kappa",    
    "lambda",    "mu",    "nu",    "xi",    
    "omicron",    "pi",    "rho",    "sigmaf",    
    "sigma",    "tau",    "upsilon",    "phi",    
    "chi",    "psi",    "omega",    "thetasym",    
    "upsih",    "piv"};
                                     
  // "fnof",
// static   String[]  HTMLsymbol2 =    {"bull",    "hellip",    
//                                     "prime",    "Prime",    "oline",    "frasl",    
//                                     "weierp",    "image",    "real",    "trade",    
//                                     "alefsym",    "larr",    "uarr",    "rarr",    
//                                     "darr",    "harr",    "crarr",    "lArr",    
//                                     "uArr",    "rArr",    "dArr",    "hArr",    
//                                     "forall",    "part",    "exist",    "empty",    
//                                     "nabla",    "isin",    "notin",    "ni",    
//                                     "prod",    "sum",    "minus",    "lowast",    
//                                     "radic",    "prop",    "infin",    "ang",    
//                                     "and",    "or",    "cap",    "cup",    "int",    
//                                     "there4",    "sim",    "cong",    "asymp",    
//                                     "ne",    "equiv",    "le",    "ge",    "sub",    
//                                     "sup",    "nsub",    "sube",    "supe",    
//                                     "oplus",    "otimes",    "perp",    "sdot",    
//                                     "lceil",    "rceil",    "lfloor",    "rfloor",    
//                                     "lang",    "rang",    "loz",    "spades",    
//                                      "clubs",    "hearts",    "diams"};
static 
{
    s_empties = new Hashtable();
    // HTML 4.0 strict DTD
    s_empties.put("AREA", "AREA");
    s_empties.put("BASE", "BASE");
    s_empties.put("BR", "BR");
    s_empties.put("COL", "COL");
    s_empties.put("HR", "HR");
    s_empties.put("IMG", "IMG");
    s_empties.put("INPUT", "INPUT");
    s_empties.put("LINK", "LINK");
    s_empties.put("META", "META");
    s_empties.put("PARAM", "PARAM");
    // HTML 4.0 loose DTD
    s_empties.put("BASEFONT", "BASEFONT");
    s_empties.put("FRAME", "FRAME");
    s_empties.put("ISINDEX", "ISINDEX");

    s_attrempties = new Hashtable();
    // HTML 4.0 strict DTD
    s_attrempties.put("checked", "checked");
    s_attrempties.put("disabled", "disabled");
    s_attrempties.put("readonly", "readonly");
    s_attrempties.put("multiple", "multiple");
    s_attrempties.put("disabled", "disabled");
    s_attrempties.put("selected", "selected");
  }
  
  /**
   * Constructor for customized encoding and doctype.
   * @param writer        The character output stream to use.
   * @param encoding      Java character encoding in use by <VAR>writer</VAR>.
   * @param doctype       String to be printed at the top of the document.
   * @param indent        Number of spaces to indent at each nesting level.
   */
  public FormatterToHTML(Writer writer, String encoding, 
                                String doctype, boolean doIndent, 
                                int indent) 
  {
    super(writer, encoding, doctype, doIndent, indent);
    this.m_attrSpecialChars = "\r\n";
  }

  /**
   * Receive notification of the end of a document.
   *
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   */
  public void endDocument ()
    throws SAXException
  {
    try
    {
      this.m_writer.write("\n");
    }
    catch(IOException ioe)
    {
      throw new SAXException(ioe);
    }
    super.endDocument();
  }
  
  String m_currentElementName = null;

  // protected boolean shouldIndent()
  // {
  //   return (!m_ispreserve && !m_isprevtext);
  // }

  /**
    * Receive notification of the beginning of an element.
    *
    * @param name The element type name.
    * @param atts The attributes attached to the element, if any.
    * @exception org.xml.sax.SAXException Any SAX exception, possibly
    *            wrapping another exception.
    * @see #endElement
    * @see org.xml.sax.AttributeList 
    */
  public void startElement (String name, AttributeList atts)
    throws SAXException
  {
    boolean savedDoIndent = m_doIndent;
    if((null != m_currentElementName) && 
       m_currentElementName.equalsIgnoreCase("IMG"))
    {
      m_doIndent = false;
    }
    // m_currentElementName = name.toUpperCase();
    m_currentElementName = name;
    if(name.equalsIgnoreCase("IMG"))
    {
      m_doIndent = false;
    }
    super.startElement(m_currentElementName, atts);
    m_doIndent = savedDoIndent;
  }

  /**
    * Receive notification of the end of an element.
    *
    * @param name The element type name
    * @exception org.xml.sax.SAXException Any SAX exception, possibly
    *            wrapping another exception.
    */
  public void endElement (String name)
    throws SAXException
  {
    try
    {
      m_currentIndent -= this.indent;
      // name = name.toUpperCase();
      boolean hasChildNodes = childNodesWereAdded();
      if (hasChildNodes) 
      {
        if (shouldIndent() && !m_currentElementName.equalsIgnoreCase("IMG"))  
          indent(this.m_writer, m_currentIndent);
        this.m_writer.write("</" + name + ">");
        m_currentElementName = name;
      }
      else
      {
        if(s_empties.get(name.toUpperCase()) == null)
        {
          this.m_writer.write("></" + name + ">");
        }
        else
        {
          this.m_writer.write(">");
        }
      }
      if (hasChildNodes) 
      {
        m_ispreserve = ((Boolean)m_preserves.pop()).booleanValue();
      }
      m_isprevtext = false;
    }
    catch(IOException ioe)
    {
      throw new SAXException(ioe);
    }
  }
  
  /**
   * Process an attribute.
   * @param   name   The name of the attribute.
   * @param   value   The value of the attribute.
   */
  protected void processAttribute(String name, String value) 
    throws SAXException
  {
    try
    {
      // String aname = name.toLowerCase();
      String aname = name;
      if(!aname.equals("xmlns") && !aname.startsWith("xmlns:"))
      {
        if (aname.equals("xml:lang"))  aname = "lang";
        String pval = prepAttrString(value, m_attrSpecialChars, this.m_encoding);
        if(s_attrempties.get(aname.toLowerCase()) == null)
        {
          this.m_writer.write(" " + aname + "=\"" +pval+ "\"");
        }
        else
        {
          if((pval.length() == 0) || pval.equalsIgnoreCase(aname))
          {
            this.m_writer.write(" " + aname);
          }
          else
          {
            this.m_writer.write(" " + aname + "=\"" +pval+ "\"");
          }
        }
      }
    }
    catch(IOException ioe)
    {
      throw new SAXException(ioe);
    }
  }
  
  private int copyEntityIntoBuf(String s, int pos)
  {
    int l = s.length();
    m_charBuf[pos++] = '&';
    for(int i= 0; i < l; i++)
    {
      m_charBuf[pos++] = s.charAt(i);
    }
    m_charBuf[pos++] = ';';
    return pos;
  }
  
  /**
   * Receive notification of character data.
   *
   * <p>The Parser will call this method to report each chunk of
   * character data.  SAX parsers may return all contiguous character
   * data in a single chunk, or they may split it into several
   * chunks; however, all of the characters in any single event
   * must come from the same external entity, so that the Locator
   * provides useful information.</p>
   *
   * <p>The application must not attempt to read from the array
   * outside of the specified range.</p>
   *
   * <p>Note that some parsers will report whitespace using the
   * ignorableWhitespace() method rather than this one (validating
   * parsers must do so).</p>
   *
   * @param chars The characters from the XML document.
   * @param start The start position in the array.
   * @param length The number of characters to read from the array.
   * @exception org.xml.sax.SAXException Any SAX exception, possibly
   *            wrapping another exception.
   * @see #ignorableWhitespace 
   * @see org.xml.sax.Locator
   */
  public void characters (char chars[], int start, int length)
    throws SAXException
  {
    if(0 == length)
      return;
    
    try
    {
      writeParentTagEnd();
      m_ispreserve = true;
      int pos = 0;
      for (int i = start;  i < length;  i ++) 
      {
        char ch = chars[i];
        int chNum = (int)ch;
        if ('<' == ch) 
        {
          pos = copyEntityIntoBuf("lt", pos);
        }
        else if ('>' == ch) 
        {
          pos = copyEntityIntoBuf("gt", pos);
        }
        else if ('&' == ch) 
        {
          pos = copyEntityIntoBuf("amp", pos);
        }
        else if((chNum >= 160) && (chNum <= 255))
        {
          pos = copyEntityIntoBuf(s_HTMLlat1[((int)ch)-160], pos);
        }
        else if((chNum >= 913) && (chNum <= 982))
        {
          pos = copyEntityIntoBuf(HTMLsymbol1[((int)ch)-913], pos);
        }
        else if (402 == ch) 
        {
          pos = copyEntityIntoBuf("fnof", pos);
        }
        else if (0xd800 <= chNum && chNum < 0xdc00) 
        {
          // UTF-16 surrogate
          int next;
          if (i+1 >= length) 
          {
            throw new SAXException("Invalid UTF-16 surrogate detected: "
              +Integer.toHexString(ch)+ " ?");
          }
          else 
          {
            next = chars[++i];
            if (!(0xdc00 <= next && next < 0xe000))
              throw new SAXException("Invalid UTF-16 surrogate detected: "
                +Integer.toHexString(ch)+" "+Integer.toHexString(next));
            next = ((ch-0xd800)<<10)+next-0xdc00+0x00010000;
          }
          m_charBuf[pos++] = '&';
          m_charBuf[pos++] = '#';
          String intStr = Integer.toString(next);
          int nIntStr = intStr.length();
          for(int k = 0; k < nIntStr; k++)
          {
            m_charBuf[pos++] = intStr.charAt(k);
          }
          m_charBuf[pos++] = ';';
        }
        else if((chNum >= 9) && (chNum <= 126))
        {
          m_charBuf[pos++] = ch;
        }
        else
        {
          m_charBuf[pos++] = '&';
          m_charBuf[pos++] = '#';
          String intStr = Integer.toString(chNum);
          int nIntStr = intStr.length();
          for(int k = 0; k < nIntStr; k++)
          {
            m_charBuf[pos++] = intStr.charAt(k);
          }
          m_charBuf[pos++] = ';';
        }
        // Use 80 as a best guess safe buffer
        if(pos > MAXSAFECHARBUF)
        {
          this.m_writer.write(m_charBuf, 0, pos);
          pos = 0;
        }
      }
      this.m_writer.write(m_charBuf, 0, pos);
      m_isprevtext = true;
    }
    catch(IOException ioe)
    {
      throw new SAXException(ioe);
    }
  }
  
  /**
    * Receive notification of cdata.
    *
    * <p>The Parser will call this method to report each chunk of
    * character data.  SAX parsers may return all contiguous character
    * data in a single chunk, or they may split it into several
    * chunks; however, all of the characters in any single event
    * must come from the same external entity, so that the Locator
    * provides useful information.</p>
    *
    * <p>The application must not attempt to read from the array
    * outside of the specified range.</p>
    *
    * <p>Note that some parsers will report whitespace using the
    * ignorableWhitespace() method rather than this one (validating
    * parsers must do so).</p>
    *
    * @param ch The characters from the XML document.
    * @param start The start position in the array.
    * @param length The number of characters to read from the array.
    * @exception org.xml.sax.SAXException Any SAX exception, possibly
    *            wrapping another exception.
    * @see #ignorableWhitespace 
    * @see org.xml.sax.Locator
    */
  public void cdata (char ch[], int start, int length)
    throws SAXException
  {
    if(m_currentElementName.equalsIgnoreCase("SCRIPT"))
    {
      try
      {
        writeParentTagEnd();
        m_ispreserve = true;
        if (shouldIndent())
          indent(this.m_writer, m_currentIndent);
        // this.m_writer.write("<![CDATA[");
        this.m_writer.write(ch, start, length);
        // this.m_writer.write("]]>");
      }
      catch(IOException ioe)
      {
        throw new SAXException(ioe);
      }
    }
    else if(m_stripCData)
    {
      try
      {
        writeParentTagEnd();
        m_ispreserve = true;
        if (shouldIndent())
          indent(this.m_writer, m_currentIndent);
        // this.m_writer.write("<![CDATA[");
        this.m_writer.write(ch, start, length);
        // this.m_writer.write("]]>");
      }
      catch(IOException ioe)
      {
        throw new SAXException(ioe);
      }
    }
    else
    {
      characters(ch, start, length);
    }
  }

  /**
   * Receive notivication of a entityReference.
   */
  public void entityReference(String name)
    throws SAXException
  {
    try
    {
      this.m_writer.write("&");
      this.m_writer.write(name);
      this.m_writer.write(";");
    }
    catch(IOException ioe)
    {
      throw new SAXException(ioe);
    }
  }
}


