/**
 * $Id: ProblemListener.java,v 1.1 1999/01/15 23:20:41 sboag Exp $
 * 
 * Copyright (c) 1998-1999 Lotus Corporation, Inc. All Rights Reserved.
 *				This software is provided without a warranty of any kind.
 * 
 * $State: Exp $
 * 
 * @author Scott Boag (scott_boag@lotus.com)
 */
package com.lotus.xsl;

import org.w3c.dom.*;

/**
 * This is the interface that the XSL processor calls when it 
 * has a problem of some kind, either an error or a warning.
 * Users should ass the XSLProcessor class to setProblemListener
 * if they wish an object instance to be called when a problem
 * event occurs.
 */
public interface ProblemListener
{
  public static final short    WARNING         = 1;
  public static final short    ERROR           = 2;

  public static final short    XMLPARSER       = 1;
  public static final short    XSLPROCESSOR    = 2;
  public static final short    QUERYENGINE     = 3;

  /**
   * Function that is called when a problem event occurs.
   * 
   * @param   where             Either and XMLPARSER, XSLPROCESSOR, or QUERYENGINE.
   * @param   classification    Either ERROR or WARNING.
   * @param   styleNode         The style tree node where the problem
   *                            occurred.  May be null.
   * @param   sourceNode        The source tree node where the problem
   *                            occurred.  May be null.
   * @param   msg               A string message explaining the problem.
   * @param   lineNo            The line number where the problem occurred,  
   *                            if it is known. May be zero.
   * @param   charOffset        The character offset where the problem,  
   *                            occurred if it is known. May be zero.
   * 
   * @return  true if the return is an ERROR, in which case
   *          exception will be thrown.  Otherwise the processor will 
   *          continue to process.
   */
  public boolean problem(short where, short classification, 
                       Node styleNode, Node sourceNode,
                       String msg, int lineNo, int charOffset);

}
