/**
 * $Id$
 * 
 * Copyright (c) 1999 Lotus Corporation, Inc. All Rights Reserved.
 *				This software is provided without a warranty of any kind.
 * 
 * $State$
 * 
 * @author Scott Boag (scott_boag@lotus.com)
 */
package com.lotus.xsl;

import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * This class does a pre-order walk of the DOM tree, calling the FormatterListener
 * interface as it goes.
 */
public class TreeWalker
{
  private DocumentHandler m_formatterListener =   null;
  
  /**
   * Constructor.
   * @param   formatterListener The implemention of the 
   * FormatterListener operation (toXMLString, digest, ...)
   */
  public TreeWalker(DocumentHandler formatterListener) 
  {
    this.m_formatterListener = formatterListener;
  }

  /**
   * Perform a pre-order traversal non-recursive style.
   */
  public void traverse(Node pos) throws SAXException 
  {
    while(null != pos)
    {     
      startNode(pos);
      
      Node nextNode = pos.getFirstChild();
      while(null == nextNode)
      {
        endNode(pos);
        nextNode = pos.getNextSibling();
        if(null == nextNode)
        {
          pos = pos.getParentNode();
          if(null == pos)
          {
            nextNode = pos;
            break;
          }
        }
      }
      pos = nextNode;
    }
  }

  /**
   * Perform a pre-order traversal non-recursive style.
   */
  public void traverse(Node pos, Node parent) throws SAXException 
  {
    while(parent != pos)
    {     
      startNode(pos);
      
      Node nextNode = pos.getFirstChild();
      while(null == nextNode)
      {
        endNode(pos);
        nextNode = pos.getNextSibling();
        if(null == nextNode)
        {
          pos = pos.getParentNode();
          if(parent == pos)
          {
            nextNode = pos;
            break;
          }
        }
      }
      pos = nextNode;
    }
  }
  
  /*
  public void traverse(Node pos) throws SAXException 
  {
    startNode(pos);
    NodeList children = pos.getChildNodes();
    if(null != children)
    {
      int nChildren = children.getLength();
      for(int i = 0; i < nChildren; i++)
      {
        traverse(children.item(i));
      }
    }
    endNode(pos);
  }
  */
  
  protected void startNode(Node node)
    throws SAXException 
  {
    switch(node.getNodeType())
    {
    case Node.COMMENT_NODE:
      {
        String data = ((Comment)node).getData();
        if(m_formatterListener instanceof FormatterListener)
        {
          ((FormatterListener)this.m_formatterListener).comment(data);
        }
      }
      break;
    case Node.DOCUMENT_FRAGMENT_NODE:
      // ??;
      break;
    case Node.DOCUMENT_NODE:
      this.m_formatterListener.startDocument();
      break;
    case Node.ELEMENT_NODE:
      NamedNodeMap atts = ((Element)node).getAttributes();
      this.m_formatterListener.startElement (node.getNodeName(), new AttList(atts));
      break;
    case Node.PROCESSING_INSTRUCTION_NODE:
      ProcessingInstruction pi = (ProcessingInstruction)node;
      this.m_formatterListener.processingInstruction(pi.getNodeName(), pi.getData());
      break;
    case Node.CDATA_SECTION_NODE:
      {
        String data = ((Text)node).getData();
        if(m_formatterListener instanceof FormatterListener)
        {
          ((FormatterListener)this.m_formatterListener).cdata(data.toCharArray(), 0, data.length());
        }
        else
        {
          this.m_formatterListener.characters(data.toCharArray(), 0, data.length());
        }
      }
      break;
    case Node.TEXT_NODE:
      {
        String data = ((Text)node).getData();
        this.m_formatterListener.characters(data.toCharArray(), 0, data.length());
      }
      break;
    case Node.ENTITY_REFERENCE_NODE:
      {
        EntityReference eref = (EntityReference)node;
        if(m_formatterListener instanceof FormatterListener)
        {
          ((FormatterListener)this.m_formatterListener).entityReference(eref.getNodeName());
        }
        else
        {
          // warning("Can not output entity to a pure SAX DocumentHandler");
        }
      }
      break;
    default:
    }
  }

  protected void endNode(Node node)
    throws SAXException 
  {
    switch(node.getNodeType())
    {
    case Node.DOCUMENT_NODE:
      this.m_formatterListener.endDocument();
      break;
    case Node.ELEMENT_NODE:
      this.m_formatterListener.endElement(node.getNodeName());
      break;
    default:
    }
  }
  
}  //TreeWalker
