/**
 * $Id: LotusXSLControl.java,v 1.3 1998/12/18 18:53:45 sboag Exp $
 * 
 * Copyright (c) 1998-1999 Lotus Corporation, Inc. All Rights Reserved.
 *				This software is provided without a warranty of any kind.
 * 
 * $State: Exp $
 * 
 * @author Scott Boag (scott_boag@lotus.com)
 */
package com.lotus.xsl.client;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import com.lotus.xsl.*;
import com.lotus.xsl.xml4j.*;

import org.w3c.dom.*;

/**
 * Class for LotusXSLControl applet.
 */
public class LotusXSLControl extends Applet
{

  /**
   * The XML Parser Liaison.
   */
  XMLParserLiaison m_liaison = null;

  /**
   * @serial
   */
  private String m_styleURL;

  /**
   * @serial
   */
  private String m_documentURL;

  // Parameter names.  To change a name of a parameter, you need only make
  // a single change.  Simply modify the value of the parameter string below.
  //--------------------------------------------------------------------------
  /**
   * @serial
   */
  private final String PARAM_styleURL = "styleURL";

  /**
   * @serial
   */
  private final String PARAM_documentURL = "documentURL";

  /**
   * @serial
   */
  private final String PARAM_parser = "parser";
  
  /**
   * @serial
   */
  private String whichParser = null;
  
  // We'll keep the DOM trees around, so tell which trees 
  // are cached.
  /**
   * @serial
   */
  private String m_styleURLOfChached = null;

  /**
   * @serial
   */
  private String m_documentURLOfChached = null;
  
  // The cached DOM trees.  The user has to call freeCache()
  // to free these.
  /**
   * @serial
   */
  private Document m_styleTree = null;

  /**
   * @serial
   */
  private Document m_docTree = null;

  // I save this for use on the worker thread, but i don't think I 
  // need to do this.
  /**
   * @serial
   */
  private URL m_codeBase = null;

  // Thread stuff for the trusted worker thread.
  /**
   * @serial
   */
  private Thread m_callThread = null;

  /**
   * @serial
   */
  private TrustedAgent m_trustedAgent = null;

  /**
   * @serial
   */
  private Thread m_trustedWorker = null;
  
  // Where the worker thread puts the HTML text.
  /**
   * @serial
   */
  private String m_htmlText = null;
  
  // Stylesheet attribute name and value that the caller can set.
  /**
   * @serial
   */
  private String m_nameOfIDAttrOfElemToModify = null;

  /**
   * @serial
   */
  private String m_elemIdToModify = null;

  /**
   * @serial
   */
  private String m_attrNameToSet = null;
  
  /**
   * @serial
   */
  private String m_attrValueToSet = null;
  
  // LotusXSLControl Class Constructor
  //--------------------------------------------------------------------------
  public LotusXSLControl()
  {
  }

  // APPLET INFO SUPPORT:
  //		The getAppletInfo() method returns a string describing the applet's
  // author, copyright date, or miscellaneous information.
  //--------------------------------------------------------------------------
  public String getAppletInfo()
  {
    return "Name: LotusXSLControl\r\n" +
      "Author: Scott Boag";
  }
  
  // PARAMETER SUPPORT
  //		The getParameterInfo() method returns an array of strings describing
  // the parameters understood by this applet.
  //
  // LotusXSLControl Parameter Information:
  //  { "Name", "Type", "Description" },
  //--------------------------------------------------------------------------
  public String[][] getParameterInfo()
  {
    String[][] info =
    {
      { PARAM_styleURL, "String", "URL to a XSL style sheet" },
      { PARAM_documentURL, "String", "URL to a XML document" },
      { PARAM_parser, "String", "Which parser to use: XML4J or ANY" },
    };
    return info;		
  } 
  
  /**
   * Standard applet initialization.
   */
  public void init()
  {
    // PARAMETER SUPPORT
    //		The following code retrieves the value of each parameter
    // specified with the <PARAM> tag and stores it in a member
    // variable.
    //----------------------------------------------------------------------
    String param;

    param = getParameter(PARAM_parser);
    whichParser = (param != null) ? param : "ANY";

    // styleURL: Parameter description
    //----------------------------------------------------------------------
    param = getParameter(PARAM_styleURL);
    if (param != null)
      setStyleURL(param);

    // documentURL: Parameter description
    //----------------------------------------------------------------------
    param = getParameter(PARAM_documentURL);
    if (param != null)
      setDocumentURL(param);
    
    m_codeBase = this.getCodeBase();
    
    initLiaison();
    
    // If you use a ResourceWizard-generated "control creator" class to
    // arrange controls in your applet, you may want to call its
    // CreateControls() method from within this method. Remove the following
    // call to resize() before adding the call to CreateControls();
    // CreateControls() does its own resizing.
    //----------------------------------------------------------------------
    resize(320, 240);
  }
    
  /**
   * Try to init the XML liaison object.
   */
  protected void initLiaison()
  {
    try
    {
      // This will throw an exception if not XML4J 2.0.X
      Class.forName("com.ibm.xml.parsers.NonValidatingDOMParser");
      Class classObj = Class.forName("com.lotus.xsl.xml4j2dom.XML4JLiaison4dom");
      m_liaison = (XMLParserLiaison)classObj.newInstance();
    }
    catch(Exception e)
    {
      try
      {
        // See if any XML4J is loaded.
        Class.forName("com.ibm.xml.parser.TXDocument");
        Class classObj = Class.forName("com.lotus.xsl.xml4j.ProcessXSL");
        m_liaison = (XMLParserLiaison)classObj.newInstance();
      }
      catch(Exception e2)
      {
        e2.printStackTrace();
      }
    }
  }

  // Place additional applet clean up code here.  destroy() is called when
  // when you applet is terminating and being unloaded.
  //-------------------------------------------------------------------------
  public void destroy()
  {
    if(null != m_trustedWorker)
    {
      m_trustedWorker.stop();
      // m_trustedWorker.destroy();
      m_trustedWorker = null;
    }
    m_styleURLOfChached = null;
    m_documentURLOfChached = null;
    m_styleTree = null;
    m_docTree = null;
  }

  // LotusXSLControl Paint Handler
  //--------------------------------------------------------------------------
  public void paint(Graphics g)
  {
  }

  //		The start() method is called when the page containing the applet
  // first appears on the screen. The AppletWizard's initial implementation
  // of this method starts execution of the applet's thread.
  //--------------------------------------------------------------------------
  public void start()
  {
    m_trustedAgent = new TrustedAgent();
    Thread currentThread = Thread.currentThread();
    m_trustedWorker = new Thread(currentThread.getThreadGroup(), m_trustedAgent);
    m_trustedWorker.start();
  }
  
  //		The stop() method is called when the page containing the applet is
  // no longer on the screen. The AppletWizard's initial implementation of
  // this method stops execution of the applet's thread.
  //--------------------------------------------------------------------------
  public void stop()
  {
    if(null != m_trustedWorker)
    {
      m_trustedWorker.stop();
      // m_trustedWorker.destroy();
      m_trustedWorker = null;
    }
    m_styleURLOfChached = null;
    m_documentURLOfChached = null;
    m_styleTree = null;
    m_docTree = null;
  }

  /**
   * Set the URI to the XSL stylesheet that will be used 
   * to display the input XML.  No processing is done yet.
   * @param valid URI string.
   */
  public void setStyleURL(String urlString)
  {        
    m_styleURL = urlString;
  }

  /**
   * Set the URI to the XML document that will be displayed 
   * via the XSL stylesheet.  No processing is done yet.
   * @param valid URI string.
   */
  public void setDocumentURL(String urlString)
  {
    m_documentURL = urlString;
  }
  
  /**
   * The processor keeps a cache of the source and 
   * style trees around, so the caller has to call 
   * freeCache() if they changed something, or if 
   * they want them garbage collected.
   */
  public void freeCache()
  {
    m_styleURLOfChached = null;
    m_documentURLOfChached = null;
    m_styleTree = null;
    m_docTree = null;
  }
  
  /**
   * Set an attribute in the stylesheet, which gives the ability 
   * to have some dynamic selection control.
   * @param nameOfIDAttrOfElemToModify The name of an attribute to search for a unique id.
   * @param elemId The unique ID to look for.
   * @param attrName Once the element is found, the name of the attribute to set.
   * @param value The value to set the attribute to.
   */
  public void setStyleSheetAttribute(String nameOfIDAttrOfElemToModify,
                                     String elemId, 
                                     String attrName, 
                                     String value)
  {
    m_nameOfIDAttrOfElemToModify = nameOfIDAttrOfElemToModify;
    m_elemIdToModify = elemId;
    m_attrNameToSet = attrName;
    m_attrValueToSet = value;
  }
  
  String m_key;
  String m_expression;

  /**
   * Push a param for the stylesheet.  For the moment, only one param is supported.
   * @param key The name of the param.
   * @param expr The expression to be evaluated.
   */
  public void setStylesheetParam(String key, String expr)
  {
    m_key = key;
    m_expression = expr;
  }

  /**
   * Given a string with markup in it, escape the markup so it 
   * can be displayed in the browser.
   */
  public String escapeString(String s)
  {
    StringBuffer sb = new StringBuffer();
    int length = s.length();
                               
    for (int i = 0;  i < length;  i ++) 
    {
      char ch = s.charAt(i);
      if ('<' == ch) 
      {
        sb.append("&lt;");
      }
      else if ('>' == ch) 
      {
        sb.append("&gt;");
      }
      else if ('&' == ch) 
      {
        sb.append("&amp;");
      }
      else if (0xd800 <= ch && ch < 0xdc00) 
      {
        // UTF-16 surrogate
        int next;
        if (i+1 >= length) 
        {
          throw new RuntimeException("Invalid UTF-16 surrogate detected: "
            +Integer.toHexString(ch)+ " ?");
        }
        else 
        {
          next = s.charAt(++i);
          if (!(0xdc00 <= next && next < 0xe000))
            throw new RuntimeException("Invalid UTF-16 surrogate detected: "
              +Integer.toHexString(ch)+" "+Integer.toHexString(next));
          next = ((ch-0xd800)<<10)+next-0xdc00+0x00010000;
        }
        sb.append("&#x");
        sb.append(Integer.toHexString(next));
        sb.append(";");
      }
      else 
      {
        sb.append(ch);
      }
    }
    return sb.toString();
  }

  
  /**
   * Having set the stylesheet URI and the input XML URI,
   * parse both XML into DOM trees, produce the result DOM
   * tree, and return the result in the form of a string.
   * This calls a trusted thread created when the applet 
   * was instantiated, because the thread that calls this 
   * function is not trusted and therefore can not access 
   * data via the URLs.
   */
  public String getHtmlText()
  {
    m_trustedAgent.m_getData = true;
    m_callThread = Thread.currentThread();
    try
    {
      synchronized(m_callThread)
      {
        m_callThread.wait();
      }
    }
    catch(InterruptedException ie)
    {
      System.out.println(ie.getMessage());
    }
    return m_htmlText;
  }
    
  /**
   * Get a DOM tree as escaped text, suitable for display 
   * in the browser.
   */
  public String getTreeAsText(Document tree)
    throws IOException
  {
    String text = null;
    try
    {
      com.lotus.xsl.Formatter formatter = (com.lotus.xsl.Formatter)m_liaison;
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      formatter.toMarkup(tree, pw, "", true);
      text = escapeString(sw.toString());
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
    return text;
  }

  /**
   * Get the XML source Tree as a text string suiteable 
   * for display in a browser.  Note that this is for display of the 
   * XML itself, not for rendering of HTML by the browser.
   * @exception Exception thrown if tree can not be converted.
   */
  public String getSourceTreeAsText()
    throws Exception
  {
    return getTreeAsText(this.m_docTree);
  }
  
  /**
   * Get the XSL style Tree as a text string suiteable 
   * for display in a browser.  Note that this is for display of the 
   * XML itself, not for rendering of HTML by the browser.
   * @exception Exception thrown if tree can not be converted.
   */
  public String getStyleTreeAsText()
    throws Exception
  {
    return getTreeAsText(this.m_styleTree);
  }

  /**
   * Get the HTML result Tree as a text string suiteable 
   * for display in a browser.  Note that this is for display of the 
   * XML itself, not for rendering of HTML by the browser.
   * @exception Exception thrown if tree can not be converted.
   */
  public String getResultTreeAsText()
    throws Exception
  {
    return escapeString(getHtmlText());
  }

  /**
   * Process a document and a stylesheet and return 
   * the text.  If one of these is null, the given
   * existing value won't be effected.
   */
  public String transformToHtml(String doc, String style)
  {
    if(null != doc)
    {
      m_documentURL = doc;
    }
    if(null != style)
    {
      m_styleURL = style;
    }
    return getHtmlText();
  }

  /**
   * Process a document and a stylesheet and return 
   * the text.  Use the xsl:stylesheet PI to find the 
   * document, if one exists.
   */
  public String transformToHtml(String doc)
  {
    if(null != doc)
    {
      m_documentURL = doc;
    }
    m_styleURL = null;
    return getHtmlText();
  }

  /**
   * Does the real transformation after the right XML processor 
   * liason has been found.
   */
  private String doTransformation(XMLParserLiaison xmlProcessorLiaison,
                                  XSLProcessor processor)
    throws XSLProcessorException, MalformedURLException, 
           FileNotFoundException, IOException, org.xml.sax.SAXException
  {
    URL documentURL = null;
    URL styleURL = null;

    if(!m_documentURL.equals(m_documentURLOfChached))
    {
      try
      {
        documentURL = new URL(m_codeBase, m_documentURL);
        
        m_docTree = xmlProcessorLiaison.parseXMLStream(documentURL);
        
        if(null == m_styleURL)
        {
          m_styleURL = processor.getStyleSheetURIfromDoc(m_docTree);
          m_styleURLOfChached = "";
        }
        
        m_documentURLOfChached = m_documentURL;
      }
      catch(MalformedURLException e)
      {
        e.printStackTrace();
        System.exit(-1);
      }
    }
    if(!m_styleURL.equals(m_styleURLOfChached))
    {
      try
      {
        styleURL = new URL(m_codeBase, m_styleURL);
        
        m_styleTree = xmlProcessorLiaison.parseXMLStream(styleURL);
        
        m_styleURLOfChached = m_styleURL;
      }
      catch(MalformedURLException e)
      {
        e.printStackTrace();
        System.exit(-1);
      }
    }

    StringWriter osw = new StringWriter();
    PrintWriter pw = new PrintWriter(osw, false);
    // xmlProcessorLiaison.SaveXMLToFile(m_resultTree, pw);

    if(null != m_elemIdToModify)
    {
      processor.modifyStylesheetAttribute(m_styleTree,
                                           m_nameOfIDAttrOfElemToModify,
                                           m_elemIdToModify, 
                                           m_attrNameToSet, 
                                           m_attrValueToSet);
    }
    
    if(null != m_key)
    {
      processor.setStylesheetParam(m_key, m_expression);
    }

    processor.process(m_docTree, m_styleTree, 
                      m_codeBase.toString(), pw);
    
    String htmlData = osw.toString();
    
    return htmlData;
  }

  /**
   * Process the transformation.
   */
  private String processTransformation()
    throws XSLProcessorException, MalformedURLException, 
           IOException, FileNotFoundException,
           org.xml.sax.SAXException
  {
    String htmlData = null;
    try
    {
      if(whichParser.trim().equals("XML4J") || whichParser.trim().equals("ANY"))
      {
        XSLProcessor processor = new XSLProcessor(m_liaison);
        htmlData = doTransformation(m_liaison, processor);
        // System.out.println("Succeeded with IBM XML processor...");
      }
      else
      {
          // System.out.println("Lotus XSLControl only works with XML4J at the moment!");
      }
    }
    catch(NoClassDefFoundError e)
    {
      System.out.println("Can not find "+whichParser+" XML Processor!!");
    }
    return htmlData;
  }
  
  /**
   * This class maintains a worker thread that that is 
   * trusted and can do things like access data.  You need 
   * this because the thread that is called by the browser 
   * is not trusted and can't access data from the URLs.
   */
  class TrustedAgent implements Runnable
  {
    public boolean m_getData = false;
    public void run()
    {
      while(true)
      {
        m_trustedWorker.yield();
        if(m_getData)
        {
          try 
          {
            m_getData = false;
            m_htmlText = null;
            m_htmlText = processTransformation();
          }
          catch(Exception e) 
          {
            e.printStackTrace();
          }
          finally
          {
            synchronized(m_callThread)
            {
              m_callThread.notify();
            }
            
          }
        }
        else
        {
          try 
          {
            m_trustedWorker.sleep(50);
          }
          catch (InterruptedException ie) 
          {
            ie.printStackTrace();
          }
        }
      }
    }
  } 
}

/**
 * $Log: LotusXSLControl.java,v $
 * Revision 1.3  1998/12/18 18:53:45  sboag
 * Stripped out Sun processor calls.
 *
 * Revision 1.2  1998/12/10 03:12:06  sboag
 * LotusXSL Version 13a
 *
 */
