/**
 * $Id: XML4JLiaison.java,v 1.4 1998/12/18 21:35:50 sboag Exp $
 * 
 * Copyright (c) 1998-1999 Lotus Corporation, Inc. All Rights Reserved.
 *				This software is provided without a warranty of any kind.
 * 
 * $State: Exp $
 * 
 * @author Scott Boag (scott_boag@lotus.com)
 */
package com.lotus.xsl.xml4j2dom;

import java.net.*;
import java.io.*;
import java.util.*;
import com.ibm.xml.parsers.*;
import com.ibm.xml.framework.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import com.lotus.xsl.*;

/**
 * A class that handles interfacing IBM's XML4J2 to the 
 * Lotus XSL Processor.
 */
public class XML4JLiaison4dom extends XMLParserLiaisonDefault
{
  /**
   * Return a string suitible for telling the user what parser is being used.
   */
  public String getParserDescription()
  {
    return "XML4J Version "+ getXML4JVersionString();
  }

  public static final String XML4J_VERSION = "XML4J 2.0.9";    
    
  /**
   * Flag to make sure we only do the version check once.
   */
  private boolean m_didVersionCheck = false;

  /**
   * Callback object that will be informed when XML-related 
   * warnings or errors occur
   */    
  private org.xml.sax.ErrorHandler m_xmlErrorHandler = null;
    
  /**
  * Constructor that takes SAX ErrorHandler as an argument. The error handler
  * is registered with the XML Parser. Any XML-related errors will be reported
  * to the calling application using this error handler.
  *
  * @param	errorHandler SAX ErrorHandler instance.
  */
  public XML4JLiaison4dom(org.xml.sax.ErrorHandler errorHandler)
  {
    m_xmlErrorHandler = errorHandler;
  }
  
  /**
   * Construct an instance.
   */
  public XML4JLiaison4dom()
  {
  }
  
  /**
   * Get XML4J's version field... we have to do this 'cause 
   * the "public static final" fields seem to get bound
   * at compile time otherwise.
   */
  private String getXML4JVersionString()
  {
    return Version.fVersion;
  }
  
  private static final int MAJOR = 1;
  private static final int MINOR = 2;
  private static final int SUBMINOR = 3;

  /**
   * Get one of the XML4J's version numbers... 
   * we have to do this 'cause the "public static final" 
   * fields seem to get bound at compile time otherwise.
   */
  private int getXML4JVersionNum(int field)
  {
    int versionNum = 0;
    String ver = getXML4JVersionString();
    StringTokenizer tokenizer = new StringTokenizer(ver, " .");
    for(int i = 0; tokenizer.hasMoreTokens(); i++)
    {
      String tok = tokenizer.nextToken();
      if(field == i)
      {
        versionNum = Integer.parseInt(tok);
      }
    }
    return versionNum;
  }
  

  /**
   * Check the XML4J version number
   */
  public void checkXML4JVersion()
    throws XSLProcessorException
  {

    // Only do this once
    if(!m_didVersionCheck)
    {
      if(!getXML4JVersionString().equals(XML4J_VERSION))
      {
        if(((getXML4JVersionNum(MAJOR) == 2) 
            && (getXML4JVersionNum(MINOR) == 0) 
            && (getXML4JVersionNum(SUBMINOR) >= 0))
           ||
           ((getXML4JVersionNum(MAJOR) == 2) 
            && (getXML4JVersionNum(MINOR) > 0))
           ||
           (getXML4JVersionNum(MAJOR) > 2))
        {
          this.m_processor.warn("Found version "+getXML4JVersionString()+" of XML4J. \n"+
                                "This version of LotusXSL is only tested with version "+
                                XML4J_VERSION+" of XML4J! \n");
        }
        else
        {
          this.m_processor.error("Found version "+getXML4JVersionString()+" of XML4J. \n"+
                                 "This version of LotusXSL doesn't work with versions less than "+
                                 XML4J_VERSION+" of XML4J! \n");
        }
      }
      m_didVersionCheck = true;
    }
  }
  
  /**
   * Parse the text pointed at by the reader as XML, and return 
   * a DOM Document interface.  Implementation of XMLParserLiaison
   * interface method.
   */
  public Document parseXMLStream(Reader reader, String filename)
    throws IOException, XSLProcessorException
  {
    Document doc = null;
    
    // NonValidatingDOMParser parser = new NonValidatingDOMParser();
    NonValidatingDOMParser parser;
    
    if(m_use_validation)
    {
      parser = new DOMParser();
      System.out.println("Validation used");
      parser.setCheckNamespace(false);
      parser.setContinueAfterFatalError(true);
    }
    else
    {
      parser = new NonValidatingDOMParser();
      parser.setNodeExpansion(com.ibm.xml.parsers.NonValidatingDOMParser.DEFERRED);
    }
    parser.setExpandEntityReferences(m_shouldExpandEntityRefs);

    try 
    {
      if(null != m_xmlErrorHandler)
      {
        parser.setErrorHandler(m_xmlErrorHandler);
      }
      else
      {
        parser.setErrorHandler(new DefaultErrorHandler());
      }
      
      InputSource source = new InputSource(reader);
      parser.parse(source);
      doc = parser.getDocument();
    }
    catch (SAXException se) 
    {
      se.printStackTrace();
    } 

    return doc;
  }
  
  /**
   * Create an empty DOM Document.  Mainly used for creating an 
   * output document.  Implementation of XMLParserLiaison
   * interface method.
   */
  public Document createDocument()
  {
    com.ibm.xml.dom.DocumentImpl doc = new com.ibm.xml.dom.DocumentImpl();
    return doc;
  }
  
  /**
   * Given an ID, return the element.
   */
  public Element getElementByID(String id, Document doc)
  {
    System.out.println("XML4JLiaison4dom does not handle IDs yet!");
    return null;
  }


  /** 
   * Tell if the node is ignorable whitespace.
   */
  public boolean isIgnorableWhitespace(Text node)
  {
    boolean isIgnorable;
    if( node instanceof com.ibm.xml.dom.TextImpl)
    {
      isIgnorable = ((com.ibm.xml.dom.TextImpl)node).isIgnorableWhitespace();
    }
    else
    {
      isIgnorable = false;
    }
    return isIgnorable;
  }
  
  /**
   * Override getNamespaceOfNode and make namespaces not work 
   * with them, since the DOM (and the XML4J DOM) is broken
   * in regards to getting the parent of an attribute.
   */
  public String getNamespaceOfNode(Node n)
  {
    String ns;
    if(Node.ATTRIBUTE_NODE == n.getNodeType())
    {
      ns = null;
    }
    else
    {
      ns = super.getNamespaceOfNode(n);
    }
    return ns;
  }
  
  /**
   * Get the parent of a node.
   */
  public Node getParentOfNode(Node node)
    throws RuntimeException
  {
    Node parent;
    int nodeType = node.getNodeType();
    if(Node.ATTRIBUTE_NODE == nodeType)
    {
      // Fix once the new XML4J is available.
      parent = super.getParentOfNode(node);
    }
    else
    {
      parent = node.getParentNode();
    }
    return parent;
  }
  
  /**
   * Command line interfact to transform the XML according to 
   * the instructions found in the XSL document.
   */
  public static void main( String argv[] )    
  {    
    String[] argv2 = new String[argv.length+2];
    argv2[0] = "-PARSER";
    argv2[1] = "com.lotus.xsl.xml4j2dom.XML4JLiaison4dom";
    int n = argv2.length;
    for(int i = 2; i < n; i++)
    {
      argv2[i] = argv[i-2];
    }
    com.lotus.xsl.Process.main(argv2);
  }
  
  /**
   * Implement SAX error handler for default reporting.
   */
  public class DefaultErrorHandler implements ErrorHandler 
  {

    DefaultErrorHandler()
    {
    }

    /**
     * Receive notification of a warning.
     *
     * <p>SAX parsers will use this method to report conditions that
     * are not errors or fatal errors as defined by the XML 1.0
     * recommendation.  The default behaviour is to take no action.</p>
     *
     * <p>The SAX parser must continue to provide normal parsing events
     * after invoking this method: it should still be possible for the
     * application to process the document through to the end.</p>
     *
     * @param exception The warning information encapsulated in a
     *                  SAX parse exception.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException 
     */
    public void warning (SAXParseException exception)
      throws SAXException
    {
      System.out.println("Parser warning: "+exception.getMessage());
    }


    /**
     * Receive notification of a recoverable error.
     *
     * <p>This corresponds to the definition of "error" in section 1.2
     * of the W3C XML 1.0 Recommendation.  For example, a validating
     * parser would use this callback to report the violation of a
     * validity constraint.  The default behaviour is to take no
     * action.</p>
     *
     * <p>The SAX parser must continue to provide normal parsing events
     * after invoking this method: it should still be possible for the
     * application to process the document through to the end.  If the
     * application cannot do so, then the parser should report a fatal
     * error even if the XML 1.0 recommendation does not require it to
     * do so.</p>
     *
     * @param exception The error information encapsulated in a
     *                  SAX parse exception.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException 
     */
    public void error (SAXParseException exception)
      throws SAXException
    {
      System.out.println("Parser error: "+exception.getMessage());
    }

    /**
     * Receive notification of a non-recoverable error.
     *
     * <p>This corresponds to the definition of "fatal error" in
     * section 1.2 of the W3C XML 1.0 Recommendation.  For example, a
     * parser would use this callback to report the violation of a
     * well-formedness constraint.</p>
     *
     * <p>The application must assume that the document is unusable
     * after the parser has invoked this method, and should continue
     * (if at all) only for the sake of collecting addition error
     * messages: in fact, SAX parsers are free to stop reporting any
     * other events once this method has been invoked.</p>
     *
     * @param exception The error information encapsulated in a
     *                  SAX parse exception.  
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.SAXParseException
     */
    public void fatalError (SAXParseException exception)
      throws SAXException
    {
      System.out.println("Parser fatal error: "+exception.getMessage());
      throw exception;
    }

  }

  
}

/**
 * $Log: XML4JLiaison.java,v $
 * Revision 1.4  1998/12/18 21:35:50  sboag
 * Removed -T option -- use lotusxsl:css-style-conversion instead.
 *
 * Revision 1.3  1998/12/11 08:43:03  sboag
 * Added Process() APIs
 *
 * Revision 1.2  1998/12/10 03:18:55  sboag
 * LotusXSL Version 13a
 *
 */
