<?xml version="1.0"?> 

<xsl:stylesheet xmlns:xsl="http://www.w3.org/XSL/Transform/1.0"
                xmlns="http://www.w3.org/TR/REC-html40" result-ns="">
  <!-- Root xsl:template - start processing here -->
  <xsl:template match="/">
    <HTML>
      <HEAD>
        <META http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
        <META http-equiv="Expires" content="0"/>
        <TITLE>        
		  <xsl:for-each select=".//teiHeader/fileDesc/titleStmt/title">
	      <DIV	style="font-size:18pt;font-family:Verdana;font-weight:bold">
		    	<xsl:apply-templates/>
		    </DIV>
		  </xsl:for-each>
		</TITLE>
      </HEAD>
      <BODY>
		<xsl:apply-templates select=".//titlePage"/><BR/>
		<HR/>
		<H1>Table of Contents</H1>
		<xsl:for-each select="tei.2//head">
      - <A href="#{@n}"><xsl:apply-templates/></A><BR/>
		</xsl:for-each>
		<xsl:apply-templates select="tei.2//body"/>
      </BODY>
    </HTML>
  </xsl:template>
  
  <xsl:template match="titlePage/docTitle/titlePart">
	<DIV	style="font-size:18pt;font-family:Verdana;font-weight:bold">
	  <xsl:apply-templates/></DIV>
  </xsl:template>
  
  <xsl:template match="titlePage/docAuthor">
    Author: <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="titlePage/docDate">
    <BR/>Document Date: <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="head">
    <A name="{@n}"><H2><xsl:apply-templates/></H2></A>
  </xsl:template>

  <xsl:template match="div1">
    <HR/>
    <xsl:apply-templates/>
  </xsl:template>
  
  <!-- <eg> contains a single short example of some technical topic 
       being discussed, e.g. a code fragment or a sample of SGML encoding. --> 
  <xsl:template match="eg">
    <CODE><PRE><xsl:apply-templates/></PRE></CODE>
  </xsl:template>

  <!-- <q> contains a quotation or apparent quotation. -->
  <xsl:template match="q">
    <Q><xsl:apply-templates/></Q>
  </xsl:template>
   
  <!-- <gi> contains a special type of identifier: an SGML 
      generic identifier, or element name. -->
  <xsl:template match="gi">
    <CODE><xsl:apply-templates/></CODE>
  </xsl:template>

  <!-- <ident> contains an identifier of some kind, e.g. 
       a variable name or the name of an SGML element or attribute.  -->
  <xsl:template match="ident">
    <CODE><xsl:apply-templates/></CODE>
  </xsl:template>

  <xsl:template match="list[@type='bulleted']">
    <UL><xsl:apply-templates/></UL>
  </xsl:template>

  <xsl:template match="list[@type='simple']">
    <UL TYPE="i"><xsl:apply-templates/></UL>
  </xsl:template>

  <xsl:template match="list[@type='gloss']">
    <DL><xsl:apply-templates/></DL>
  </xsl:template>

  <xsl:template match="list[@type='gloss']/label"><DT><B><xsl:apply-templates/></B></DT></xsl:template>

  <xsl:template match="list[@type='gloss']/item"><DD><xsl:apply-templates/></DD></xsl:template>

  <xsl:template match="list[@type='ordered']">
    <OL><xsl:apply-templates/></OL>
  </xsl:template>

  <xsl:template match="list/item" priority="-1">
    <LI><xsl:apply-templates/></LI>
  </xsl:template>
  
  <!-- <code> contains a short fragment of code in some 
  formal language (often a programming language). 
  (He uses it for URLs, without identifying it as such via 
  and attribute.  This should be fixed, I think. -->
  <xsl:template match="code">
    <A HREF="{.}"><xsl:apply-templates/></A>
  </xsl:template>  

  <!-- <emph> marks words or phrases which are stressed or emphasized 
       for linguistic or rhetorical effect. --> 
  <xsl:template match="emph">
    <EM><xsl:apply-templates/></EM>
  </xsl:template>

  <!-- <soCalled> ... --> 
  <xsl:template match="soCalled">
    <I><xsl:apply-templates/></I>
  </xsl:template>

  <!-- <term> contains a single-word, multi-word or symbolic 
       designation which is regarded as a technical term. --> 
  <xsl:template match="term">
    <DFN><xsl:apply-templates/></DFN>
  </xsl:template>

  <xsl:template match="note">
    <SPAN style="border:1px solid black;color:teal;">[NOTE: <xsl:apply-templates/>]</SPAN>
  </xsl:template>

  <xsl:template match="title">
    <SPAN style="text-decoration:underline;"><xsl:apply-templates/></SPAN>
  </xsl:template>

  <xsl:template match="p">
    <P><xsl:apply-templates/></P>
  </xsl:template>

  <xsl:template match="*" priority="-5">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="teiHeader//*">
  </xsl:template>
  
</xsl:stylesheet> 
