/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package SAXCount;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;

import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.ParserFactory;
import org.w3c.dom.Document;

/**
 *
 * @version Revision: 87 1.8 samples/SAXCount/SAXCount.java, samples, xml4j2, xml4j2_0_13 
 */

public class SAXCount {

    public static void main(String args[])
        throws Exception
    {
        String parserClass = "com.ibm.xml.parsers.SAXParser";
        String xmlFile = null;
        URL u = null;
        boolean warmup = false;

        //
        // Process the command line arguments.
        //
        try {
            for (int i=0; i<args.length; i++) {
                if (args[i].equals("-w")) {
                    warmup = true;
                    continue;
                }
                if (args[i].equals("-p")) {
                    ++i;
                    parserClass = args[i];
                    continue;
                }
                if (u != null) throw new Exception();
                u = createURL(args[i]);
            }
        } catch (Exception e) {
            usage();
            System.exit(-1);
        }
        if (u==null) {
            usage();
            System.exit(-1);
        }

        //
        // Create the parser and SAX handlers.
        //
        xmlFile = u.toString();
        Parser parser = ParserFactory.makeParser(parserClass);
        SAXCountHandlers handler = new SAXCountHandlers();
        parser.setDocumentHandler(handler);
        parser.setErrorHandler(handler);
        if (warmup) {
            parser.parse(xmlFile);
            handler.reset();
        }
        //
        // Parse the XML file.
        //
        long  startTime = System.currentTimeMillis();
        try {
            parser.parse(xmlFile);
        } catch (SAXException sx) {
            Exception ex = sx.getException();
            if (ex != null)
                ex.printStackTrace();
            else
                sx.printStackTrace();
        }
        long  duration = System.currentTimeMillis() - startTime;

        //
        // Display summary of file contents.
        //
        String label = xmlFile;
        int index = label.lastIndexOf('\\');
        if (index == -1)
            index = label.lastIndexOf('/');
        if (index != -1)
            label = label.substring(index + 1);
        if (parser instanceof com.ibm.xml.parsers.SAXParser) {
            System.out.println(label + ": " + duration + " ms (" +
                handler.getElementCount() + " elems, " +
                handler.getAttrCount() + " attrs, " +
                handler.getSpaceCount() + " spaces, " +
                handler.getCharacterCount() + " chars)");
        } else if (parser instanceof com.ibm.xml.parsers.NonValidatingDOMParser) {
            Document doc = ((com.ibm.xml.parsers.NonValidatingDOMParser)parser).getDocument();
            int elementCount = doc.getElementsByTagName("*").getLength();
            System.out.println(label + ": " + duration + " ms (" + elementCount + " elems)");
        } else if (parser instanceof com.ibm.xml.framework.XMLParser) {
            System.out.println(label + ": " + duration + " ms");
        } else {
            System.out.println(label + ": " + duration + " ms (" +
                handler.getElementCount() + " elems, " +
                handler.getAttrCount() + " attrs, " +
                handler.getSpaceCount() + " spaces, " +
                handler.getCharacterCount() + " chars)");
        }
    }


    static void usage() {
        System.out.println("Usage: java SAXCount [-p <Parser class name>]" +
                           "(<XML file> | <URL to XML file>)");
    }
    
    
    //
    // Create a URL object from either a URL string or a plain file name.
    //
    static URL createURL(String name) throws Exception {
        try {
            URL u = new URL(name);
            return u;
        } catch (MalformedURLException ex) {
        }
        URL u = new URL("file:" + new File(name).getAbsolutePath());
        return u;
    };
    
}
