/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package SAXCount;

import XJParseHandler;
import org.xml.sax.Parser;
import org.xml.sax.HandlerBase;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;

/**
 *
 * @version Revision: 88 1.7 samples/SAXCount/SAXCountHandlers.java, samples, xml4j2, xml4j2_0_13 
 */

public class SAXCountHandlers extends HandlerBase implements XJParseHandler {
    //
    //
    //
    private Parser fParser;
    private int fElementCount;
    private int fAttrCount;
    private int fCharacterCount;
    private int fSpaceCount;
    /**
     *
     */
    public SAXCountHandlers() {
    }
    /**
     *
     */
    public void registerHandlers(Parser parser) {
        fParser = parser;
        parser.setDocumentHandler(this);
        parser.setErrorHandler(this);
    }
    /**
     *
     */
    public void reset() {
        fElementCount = 0;
        fAttrCount = 0;
        fCharacterCount = 0;
        fSpaceCount = 0;
    }
    /**
     *
     */
    public void printResults(String label, long duration) {
        if (fParser instanceof com.ibm.xml.parsers.SAXParser) {
            System.out.println(label + ": " + duration + " ms (" +
                getElementCount() + " elems, " +
                getAttrCount() + " attrs, " +
                getSpaceCount() + " spaces, " +
                getCharacterCount() + " chars)");
        } else if (fParser instanceof com.ibm.xml.parsers.NonValidatingDOMParser) {
            Document doc = ((com.ibm.xml.parsers.NonValidatingDOMParser)fParser).getDocument();
            int elementCount = doc.getElementsByTagName("*").getLength();
            System.out.println(label + ": " + duration + " ms (" + elementCount + " elems)");
        } else if (fParser instanceof com.ibm.xml.framework.XMLParser) {
            System.out.println(label + ": " + duration + " ms");
        } else {
            System.out.println(label + ": " + duration + " ms (" +
                getElementCount() + " elems, " +
                getAttrCount() + " attrs, " +
                getSpaceCount() + " spaces, " +
                getCharacterCount() + " chars)");
        }
    }

    int getElementCount() { return (fElementCount); }
    int getAttrCount() { return (fAttrCount); }
    int getCharacterCount() { return (fCharacterCount); }
    int getSpaceCount() { return (fSpaceCount); }

    public void startElement(String name, AttributeList atts) {
        fElementCount++;
        fAttrCount += atts.getLength();
    }

    public void characters(char ch[], int start, int length) {
        fCharacterCount += length;
    }

    public void ignorableWhitespace (char ch[], int start, int length) {
        fSpaceCount += length;
    }

    public void warning(SAXParseException e) throws SAXException {
        System.err.println("Warning at (file " + e.getSystemId() + ", line " +
            e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
    }
    public void error(SAXParseException e) throws SAXException {
        System.err.println("Error at (file " + e.getSystemId() + ", line " +
            e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
        // no op
    }
    public void fatalError(SAXParseException e) throws SAXException {
        System.err.println("Fatal Error at (file " + e.getSystemId() +
            ", line " + e.getLineNumber() + ", char " + e.getColumnNumber() + "): " + e.getMessage());
//        throw e;
    }
}
