/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

import org.xml.sax.Parser;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.ParserFactory;

/**
 *
 * @version Revision: 60 1.4 samples/XJParse.java, parser, xml4j2, xml4j2_0_13 
 */

public class XJParse {

    public static void main(String args[])
        throws Exception
    {
        String parserClass = "com.ibm.xml.parsers.SAXParser";
        String handlerClass = "SAXCount.SAXCountHandlers";
        String documentClass = null;
        boolean warmup = false;
        String xmlFile = null;

        //
        // Process the command line arguments.
        //
        try {
            for (int i=0; i<args.length; i++) {
                if (args[i].equals("-p")) {
                    ++i;
                    parserClass = args[i];
                    continue;
                }
                if (args[i].equals("-h")) {
                    ++i;
                    handlerClass = args[i];
                    continue;
                }
                if (args[i].equals("-d")) {
                    ++i;
                    documentClass = args[i];
                    continue;
                }
                if (args[i].equals("-w")) {
                    warmup = true;
                    continue;
                }
                xmlFile = args[i];
            }
        } catch (Exception e) {
            usage();
            System.exit(-1);
        }
        if (xmlFile == null) {
            usage();
            System.exit(-1);
        }

        //
        // Create the parser and handlers.
        //
        Parser parser = ParserFactory.makeParser(parserClass);
        XJParseHandler handler = null;
        try {
            handler = (XJParseHandler)(Class.forName(handlerClass).newInstance());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        handler.registerHandlers(parser);
        //
        // Set DOM Document class is requested.
        //
        if (documentClass != null)
            ((com.ibm.xml.parsers.NonValidatingDOMParser)parser).setDocumentClass(documentClass);
        //
        // Parse the XML file.
        //
        if (warmup) {
            try {
                parser.parse(xmlFile);
            } catch (SAXException sx) {
                Exception ex = sx.getException();
                if (ex != null)
                    ex.printStackTrace();
                else
                    sx.printStackTrace();
            }
            handler.reset();
        }
        long startTime = System.currentTimeMillis();
        try {
            parser.parse(xmlFile);
        } catch (SAXException sx) {
            Exception ex = sx.getException();
            if (ex != null)
                ex.printStackTrace();
            else
                sx.printStackTrace();
        }
        long duration = System.currentTimeMillis() - startTime;

        //
        // Display summary of file contents.
        //
        String label = xmlFile;
        int index = label.lastIndexOf('\\');
        if (index == -1)
            index = label.lastIndexOf('/');
        if (index != -1)
            label = label.substring(index + 1);
        handler.printResults(label, duration);
    }


    static void usage() {
        System.out.println("Usage: java XJParse <options> (<XML file> | <URL to XML file>)");
        System.out.println("Options:");
        System.out.println("  -p <parser class name>    (default is com.ibm.xml.parsers.SAXParser)");
        System.out.println("  -h <handler class name>   (default is SAXCount.SAXCountHandlers)");
        System.out.println("  -d <document class name>  (default is com.ibm.domimpl.DocumentImpl)");
        System.out.println("  -w                        (warmup the parser before timing)");
    }

}
