/*
 * (C) Copyright IBM Corp. 1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package dom.wrappers;

import dom.DOMParserWrapper;

import com.ibm.xml.parser.Parser;
import com.ibm.xml.parser.ErrorListener;

import org.w3c.dom.Document;

/**
 * Wraps the XML4J 2 TX-compat parser.
 *
 * @version Revision: 26 1.2 samples/dom/wrappers/TXParser.java, samples, xml4j2, xml4j2_0_13 
 */
public class TXParser
    implements DOMParserWrapper, ErrorListener {

    //
    // DOMParserWrapper methods
    //

    /** Parses the specified URI and returns the document. */
    public Document parse(String uri) throws Exception {

        Parser parser = new Parser(uri, this, null);
        parser.readStream(parser.getInputStream(null, null, uri));
        return parser.getDocument();

    } // parse(String):Document

    //
    // ErrorListener methods
    //

    /** Error. */
    public int error(String file, int line, int col, Object key, String msg) {

        String type = "???";
        if (key != null && key instanceof String) {
            String key_str = (String)key;
            if (key_str.startsWith("W_")) {
                type = "Warning";
            }
            else if (key_str.startsWith("V_")) {
                type = "Error";
            }
            else if (key_str.startsWith("E_")) {
                type = "Fatal Error";
            }
        }

        System.err.println("["+type+"] "+
                           getLocationString(file, line, col)+": "+
                           msg);

        return 1;
    }

    //
    // Private methods
    //

    /** Returns a string of the location. */
    private String getLocationString(String file, int line, int col) {
        
        if (file == null) {
            return ":"+line+':'+col;
        }
        return file+':'+line+':'+col;

    } // getLocationString(SAXParseException):String

} // class TXParser
