/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.dom;

import org.w3c.dom.DOMException;

/**
 * DOMExceptions are thrown when one of the
 * DOM implementation classes discovers an error condition. The
 * architecture is defined in terms of the ExceptionCode, but in the
 * standard Java API this has been extended so the exception may also
 * carry a text string. Standard exception codes are:
 * <UL>
 * <LI>
 * INDEX_SIZE_ERR: index or size is negative, or greater than the
 * allowed value.
 * <LI>
 * WSTRING_SIZE_ERR: the specified range of text does not fit into the
 * string representation used in this implementation. Only arises in
 * implementations whose strings have a maximum-size limit. The DOM
 * provides workarounds to support those languages, generally by
 * having the DOM extract a managable substring on their behalf.
 * <LI>
 * HIERARCHY_REQUEST_ERR: user tried to insert a node somewhere it
 * doesn't belong... such as adding a child to a Text node.
 * <LI>
 * WRONG_DOCUMENT_ERR: a node is used in a different document than the
 * one that created it (that doesn't support it). Currently, the DOM
 * doesn't promise that nodes can be moved from document to document
 * even if they're both using the same underlying implementation.
 * Level 2 of the DOM spec may address this.
 * <LI>
 * INVALID_NAME_ERR: an invalid name is specified.
 * <LI>
 * NO_DATA_ALLOWED_ERR: data is specified for a node which does not
 * support data.
 * <LI>
 * NO_MODIFICATION_ALLOWED_ERR: an attempt is made to modify an object
 * where modifications are not allowed.
 * <LI>
 * NOT_FOUND_ERR: an attempt was made to reference a node in a context
 * where it does not exist.
 * <LI>
 * NOT_SUPPORTED_ERR: the implementation does not support the type of
 * object requested.
 * <LI>
 * INUSE_ATTRIBUTE_ERR: an attempt is made to add an attribute object
 * that is already in use elsewhere.
 * </UL>
 *
 * @version Revision: 58 1.3 src/com/ibm/xml/dom/DOMExceptionImpl.java, parser, xml4j2, xml4j2_0_13 
 * @since  PR-DOM-Level-1-19980818.
 */
public class DOMExceptionImpl 
    extends DOMException {

    //
    // Constructors
    //

    /** Constructs a dom exception. */
    public DOMExceptionImpl(short code, String message) {
	    super(code, message);
    }    

} // class DOMExceptionImpl
