/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 * Parameter object for the (@link ElementDeclPool#addAttDef) method.
 *
 * @author  Glenn Marcy
 * @version Revision: 41 1.5 src/com/ibm/xml/framework/AttDef.java, parser, xml4j2, xml4j2_0_13 
 * @see ElementDeclPool#addAttDef
 */
public final class AttDef {
    //
    // attType
    //
    public static final int
        CDATA           = 0,
        ID              = 1,
        IDREF           = 2,
        IDREFS          = 3,
        ENTITY          = 4,
        ENTITIES        = 5,
        NMTOKEN         = 6,
        NMTOKENS        = 7,
        NOTATION        = 8,
        ENUMERATION     = 9;
    //
    // Keep this array in-sync with the ATTTYPE definitions
    //
    private static final String[] fgAttTypeString = {
        "CDATA",                        // 0
        "ID",                           // 1
        "IDREF",                        // 2
        "IDREFS",                       // 3
        "ENTITY",                       // 4
        "ENTITIES",                     // 5
        "NMTOKEN",                      // 6
        "NMTOKENS",                     // 7
        "NOTATION",                     // 8
        "ENUMERATION"                   // 9 (replaced with "NMTOKEN" by SAX AttributeList handler)
    };
    //
    // attDefaultType
    //
    public static final int
        DEFAULT     = 1,                // AttValue
        REQUIRED    = 2,                // #REQUIRED
        IMPLIED     = 3,                // #IMPLIED
        FIXED       = 4;                // #FIXED AttValue
    //
    // Public instance variables - fill these in for each call to addAttDef.
    //
    public int attName;                 // Attr Name
    public int attType;                 // AttType - CDATA, ID, IDREF, IDREFS, NMTOKEN, NMTOKENS,
                                        //             ENTITY, ENTITIES, NOTATION or ENUMERATION
    public int enumeration;             // enumeration - for NOTATION and ENUMERATION
    public int attDefaultType;          // DefaultDecl - REQUIRED, IMPLIED, FIXED (or DEFAULT)
    public int attValue;                // default AttValue
    //
    // Utility method to get AttType string (for SAX.AttributeList)
    //
    public static final String getAttTypeString(int attrType) {
        try {
            return fgAttTypeString[attrType];
        } catch (ArrayIndexOutOfBoundsException ex) {
            return null;
        }
    }
};
