/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

import org.xml.sax.AttributeList;

/**
 *
 * @version Revision: 42 1.6 src/com/ibm/xml/framework/AttrPool.java, parser, xml4j2, xml4j2_0_13 
 */

public interface AttrPool {
    //
    // Reset this pool for use during the next invocation of the parser.
    //
    public void reset(ParserState parserState);
    //
    // Return a pool for use during the next invocation of the parser.
    //
    public AttrPool resetOrCopy(ParserState parserState);
    //
    //
    //
    public int addAttr(Attr attr, int firstAttrIndex) throws Exception;
    public void setIsLastAttr(int attrIndex);
    //
    //
    //
    public int getAttrName(int attrIndex);
    public int getAttType(int attrIndex);
    public int getAttValue(int attrIndex);
    public boolean isSpecified(int attrIndex);
    public boolean isLastAttr(int attrIndex);
    //
    // Release resources use by this attribute list.
    //
    public void releaseAttrList(int attrListIndex);
    //
    // AttributeList support
    //
    public AttributeList getAttributeList(int attrListIndex);
    
    /** 
     * Returns the index of the first attribute in the specified
     * attribute list or -1 if the index is invalid.
     *
     * @param attrListIndex The attribute list index.
     */
    public int getFirstAttr(int attrListIndex);

    /**
     * Returns the next index of the specified attribute or -1 if
     * there is no next index.
     *
     * @param attrIndex The attribute index.
     *
     * @see #isLastAttr
     */
    public int getNextAttr(int attrIndex);
}
