/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * Interface for implementing basic catalog support in the parser.
 * To implement and use a Catalog, implement this interface and
 * install your catalog instance as the EntityResolver in the
 * parser's entity handler. For example:
 * <pre>
 *     XMLParser parser = new AnyParser();
 *     <font color="blue">parser.getEntityHandler().setEntityResolver(new MyCatalog());</font>
 * </pre>
 * 
 * @author  Andy Clark, IBM
 * @version Revision: 65 1.2 src/com/ibm/xml/framework/Catalog.java, parser, xml4j2, xml4j2_0_13 
 *
 * @see org.xml.sax.EntityResolver
 * @see com.ibm.xml.framework.XMLEntityHandler#setEntityResolver
 */
public interface Catalog 
    extends EntityResolver
    {

    /**
     * Loads the catalog stream specified by the given input source and
     * appends the contents to the catalog.
     *
     * @param source The catalog source.
     *
     * @exception java.lang.Exception Throws an exception if an error
     *                                occurs while loading the catalog source.
     */
    public void loadCatalog(InputSource source) throws Exception;

    } // interface Catalog
