/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 *
 * @version Revision: 40 1.7 src/com/ibm/xml/framework/ElementDeclPool.java, parser, xml4j2, xml4j2_0_13 
 */
public interface ElementDeclPool {
    /**
     *  Reset this pool for use during the next invocation of the parser.
     */
    public void reset(ParserState parserState);
    /**
     *  Return a pool for use during the next invocation of the parser.
     */
    public ElementDeclPool resetOrCopy(ParserState parserState);
    /**
     *  Set the root element
     */
    public void setRootElement(int elementIndex);
    /**
     *  Get the root element index.
     */
    public int getRootElement();
    //
    // element entries
    //
    
    /** Return an element index, given an element name index. */
    public int getElement(int elementNameIndex);
    /** Add an element and return the elementIndex, given an elementNameIndex. */
    public int addElement(int elementNameIndex);
    /** Add an ElementDecl. You must  build up a tree of ContentSpecNodes
     *  using addContentSpecNode and set ElementDecl.contentSpec to,
     *  the return value of the root addContentSpecNode call.
     */
    public int addElementDecl(ElementDecl decl);
    /** Return an index which can be used by StringPool#toString */
    public int getElementName(int elementIndex);
    /** Return an int corresponding to a CONTENTSPEC_* constant in ElementDecl */
    public int getContentSpecType(int elementIndex);
    /** Return a contentSpecIndex which can be used by getContentSpecNode */ 
    public int getContentSpec(int elementIndex);
    /** Return a String representation of the content model for an elementIndex. */ 
    public String getContentSpecAsString(int elementIndex);
    /** Return a ContentModel, for validation. */ 
    public ContentModel getContentModel(int elementIndex);
    /** Set a ContentModel, for validation. */ 
    public void setContentModel(int elementIndex, ContentModel cm);
    //
    // contentspec entries
    //
    public int addContentSpecNode(ContentSpecNode csn);
    /** Given a contentSpecIndex and a reference to a ContentSpecNode, this 
     *  function will fill in the public fields of the ContentSpecNode, 
     *  which can be accessed via the input reference, after the call.
     */
    public void getContentSpecNode(int contentSpecIndex, ContentSpecNode csn);
    
    /** Return the content spec node as a String. */ 
    public String getContentSpecNodeAsString(int contentSpecIndex);
    //
    // attribute list interfaces
    //
    /** Create and populate an AttDef, and this function will add it,
     * and return the new attDefIndex for it.
     */
    public int addAttDef(int elementIndex, AttDef attDef) throws Exception;
    /** Returns an index into the StringPool of the attribute name */
    public int getAttDef(int elementNameIndex, int attrNameIndex);
    /** Returns an index into the StringPool of the attribute name */
    public int getAttName(int attDefIndex);
    /** Returns an index into the StringPool of the attribute value */
    public int getAttValue(int attDefIndex);
    /** Returns an int corresponding to the com.ibm.xml.framework.AttDef 
     *  AttType constant. 
     */
    public int getAttType(int attDefIndex);
    /** Returns an int corresponding to the com.ibm.xml.framework.AttDef 
     *  AttDefaultType constant. 
     */
    public int getAttDefaultType(int attDefIndex);
    /** Returns an index into the StringPool's stringsInList. The
     *  StringPool#stringsInList will return an array of int values
     *  which can be turned into Strings using StringPool#toString.
     *  @see StringPool#stringsInList
     */
    public int getEnumeration(int attDefIndex);
    public int addDefaultAttributes(int elementIndex, AttrPool attrPool, int firstAttrIndex, int lastAttrIndex) throws Exception;
    //
    // ID attribute definitions and references (IDREF)
    //
    public boolean addId(int idIndex, int elementNameIndex);
    public void addIdRef(int idIndex, int elementNameIndex);
    public void checkIdRefs() throws Exception;
    //
    // Check simple namespace constraints
    //
    public void checkNamespace(int elementIndex, int firstAttrIndex) throws Exception;

    /** 
     * Checks that all declared elements refer to declared elements
     * in their content models. This method calls out to the error
     * handler to indicate warnings.
     */
    public void checkDeclaredElements() throws Exception;

}
