/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 *
 * @version Revision: 46 1.6 src/com/ibm/xml/framework/EntityPool.java, parser, xml4j2, xml4j2_0_13 
 */
public interface EntityPool {
    //
    // Reset this pool for use during the next invocation of the parser.
    //
    public void reset(ParserState parserState);
    //
    // Return a pool for use during the next invocation of the parser.
    //
    public EntityPool resetOrCopy(ParserState parserState);
    //
    //
    //
    public int addEntityDecl(EntityDecl decl);
    public int addNotationDecl(NotationDecl decl);
    //
    //
    //
    public int lookupEntity(int entityNameIndex);
    public boolean isExternal(int entityIndex);
    public boolean isUnparsed(int entityIndex);
    public int getEntityName(int entityIndex);
    public int getEntityValue(int entityIndex);
    public int getPublicId(int entityIndex);
    public int getSystemId(int entityIndex);
    public int getNotationName(int index);
    public int lookupNotation(int notationNameIndex);

    /**
     * Checks that all of the declared unparsed entites refer
     * to declared notations. This method emits an error for each
     * unparsed entity that fails the check.
     *
     * @exception java.lang.Exception Thrown if error handler
     *                                throws exception to signal
     *                                a fatal error.
     */
    public void checkUnparsedEntities() throws Exception;

}
