/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

import com.ibm.xml.internal.DefaultStringPool;
import com.ibm.xml.internal.DefaultAttrPool;
import com.ibm.xml.internal.DefaultEntityPool;
import com.ibm.xml.internal.DefaultElementDeclPool;

/**
 * This class is used to determine which implementation objects are to be used during
 * a single use of the parser.  The information described here may change between each
 * use of the parser.
 *
 * @author  Glenn Marcy
 * @version Revision: 63 1.3 src/com/ibm/xml/framework/ParserState.java, parser, xml4j2, xml4j2_0_13 
 * @see XMLParser
 */
public final class ParserState
{
    //
    //
    //
    private static final boolean USE_CHAR_READER_FOR_UTF8 = true;
    //
    // Instance variables
    //
    private XMLParser fParser = null;
    //
    // Pools
    //
    private StringPool fStringPool = null;
    private AttrPool fAttrPool = null;
    private EntityPool fEntityPool = null;
    private ElementDeclPool fElementDeclPool = null;
    /**
     * Constructor
     */
    ParserState(XMLParser parser) {
        fParser = parser;
    }
    /**
     * Copy constructor
     */
    ParserState(ParserState parserState) {
        fParser = parserState.fParser;
        fStringPool = parserState.fStringPool.resetOrCopy(this);
        fAttrPool = parserState.fAttrPool.resetOrCopy(this);
        fEntityPool = parserState.fEntityPool.resetOrCopy(this);
        fElementDeclPool = parserState.fElementDeclPool.resetOrCopy(this);
    }
    /**
     *
     */
    void reset() {
        fStringPool.reset(this);
        fAttrPool.reset(this);
        fEntityPool.reset(this);
        fElementDeclPool.reset(this);
    }
    /**
     *
     */
    ParserState resetOrCopy() {
        return new ParserState(this);
    }
    /**
     *
     */
    public XMLScanner getScanner() {
        return fParser.getScanner();
    }
    /**
     *
     */
    public boolean getAllowJavaEncodingName() {
        return fParser.getAllowJavaEncodingName();
    }
    /**
     *
     */
    public boolean getWarningOnDuplicateAttDef() {
        return fParser.getWarningOnDuplicateAttDef();
    }
    /**
     *
     */
    public boolean getCheckNamespace() {
        return fParser.getCheckNamespace();
    }
    /**
     *
     */
    public boolean getUseCharReaderForUTF8() {
        return USE_CHAR_READER_FOR_UTF8;
    }
    /**
     *
     */
    public XMLDocumentHandler getDocumentHandler() {
        return fParser.getDocumentHandler();
    }
    /**
     *
     */
    public XMLDocumentTypeHandler getDocumentTypeHandler() {
        return fParser.getDocumentTypeHandler();
    }
    /**
     *
     */
    public XMLEntityHandler getEntityHandler() {
        return fParser.getEntityHandler();
    }
    /**
     *
     */
    public XMLErrorHandler getErrorHandler() {
        return fParser.getErrorHandler();
    }
    /**
     *
     */
    public XMLValidationHandler getValidationHandler() {
        return fParser.getValidationHandler();
    }
    /**
     *
     */
    public StringPool cacheStringPool() {
        return fStringPool;
    }
    /**
     *
     */
    public StringPool getStringPool() {
        return fStringPool;
    }
    /**
     *
     */
    public void setStringPool(StringPool pool) {
        fStringPool = pool;
    }
    /**
     *
     */
    public void useDefaultStringPool() {
        fStringPool = new DefaultStringPool(this);
    }
    /**
     *
     */
    public AttrPool cacheAttrPool() {
        return fAttrPool;
    }
    /**
     *
     */
    public AttrPool getAttrPool() {
        return fAttrPool;
    }
    /**
     *
     */
    public void setAttrPool(AttrPool pool) {
        fAttrPool = pool;
    }
    /**
     *
     */
    public void useDefaultAttrPool() {
        fAttrPool = new DefaultAttrPool(this);
    }
    /**
     *
     */
    public EntityPool getEntityPool() {
        return fEntityPool;
    }
    /**
     *
     */
    public void setEntityPool(EntityPool pool) {
        fEntityPool = pool;
    }
    /**
     *
     */
    public void useDefaultEntityPool() {
        fEntityPool = new DefaultEntityPool(this, true);
    }
    /**
     *
     */
    public ElementDeclPool cacheElementDeclPool() {
        return fElementDeclPool;
    }
    /**
     *
     */
    public ElementDeclPool getElementDeclPool() {
        return fElementDeclPool;
    }
    /**
     *
     */
    public void setElementDeclPool(ElementDeclPool pool) {
        fElementDeclPool = pool;
    }
    /**
     *
     */
    public void useDefaultElementDeclPool() {
        fElementDeclPool = new DefaultElementDeclPool(this);
    }
}
