/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 *
 * @version Revision: 48 1.6 src/com/ibm/xml/framework/StringPool.java, parser, xml4j2, xml4j2_0_13 
 */
public interface StringPool {
    /**
     *  Reset this pool for use during the next invocation of the parser.
     */
    public void reset(ParserState parserState);
    /**
     *  Return a pool for use during the next invocation of the parser.
     */
    public StringPool resetOrCopy(ParserState parserState);
    /**
     * Add string.  Assign a new index to this string.  Two matching strings have
     * different string indexes. Used for most "values".
     */
    public int addString(String s);
    public int addString(StringProducer stringProducer, int offset, int length);
    /**
     * Add symbol.  Assign a string index to each unique symbol string.  Two matching
     * symbols have the same string index.  Used for Element, Attribute and Entity names.
     */
    public int addSymbol(String s);
    public int addSymbol(StringProducer stringProducer, int offset, int length, int hashcode);
    /**
     * Maintain a list of strings.  Used for name, nmtoken, notation and enumeration lists.
     */
    public int startStringList();
    /**
     * Add a string from the StringPool to a "StringList".
     */
    public boolean addStringToList(int stringListIndex, int stringIndex);
    /**
     * Finish a "StringList."
     */
    public void finishStringList(int stringListIndex);
    /**
     * Return the length of a "StringList."
     */
    public int stringListLength(int stringListIndex);
    /**
     * Return true if a stringIndex (from the StringPool) is in a "StringList."
     */
    public boolean stringInList(int stringListIndex, int stringIndex);
    /** 
     *  Given a stringListIndex return an array populated by
     *  indexes of strings in the StringPool. 
     */
    public int [] stringsInList(int stringListIndex) ;
    /**
     * Release index.  All resources associated with <code>stringIndex</code> will be
     * released and the index can be reused.
     */
    public void releaseString(int stringIndex);
    /**
     * Get String value.  Cache the result.  Used for both Strings and Symbols
     */
    public String toString(int stringIndex);
    /**
     * Get String value.  All resources associated with <code>stringIndex</code> will be
     * released and the index can be reused.  Optimized form of <code>toString</code> followed
     * by <code>releaseString</code> that eliminates an wasteful caching operation.
     */
    public String orphanString(int stringIndex);
}
