/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

import java.io.IOException;
import org.xml.sax.InputSource;

/**
 *
 * @version Revision: 50 1.5 src/com/ibm/xml/framework/XMLDeclRecognizer.java, parser, xml4j2, xml4j2_0_13 
 */
public abstract class XMLDeclRecognizer {
    //
    // Subclasses override this method to support recognizing their encodings.
    //
    public abstract XMLReader recognize(ParserState parserState, InputSource source,
                                        ChunkyByteArray data, boolean xmlDecl) throws Exception;
    //
    // Support for getting the value of an EncodingDecl using an XMLReader.
    //
    // This is the minimal logic from the scanner to recognize and XMLDecl or TextDecl using
    // the XMLReader interface.
    //
    // From the standard:
    //
    // [23] XMLDecl ::= '<?xml' VersionInfo EncodingDecl? SDDecl? S? '?>'
    // [24] VersionInfo ::= S 'version' Eq (' VersionNum ' | " VersionNum ")
    // [80] EncodingDecl ::= S 'encoding' Eq ('"' EncName '"' |  "'" EncName "'" )
    // [81] EncName ::= [A-Za-z] ([A-Za-z0-9._] | '-')*
    // [77] TextDecl ::= '<?xml' VersionInfo? EncodingDecl S? '?>'
    //
    protected final int prescanXMLDeclOrTextDecl(XMLReader reader, boolean xmlDecl) throws Exception
    {
        if (!reader.skippedChar('<'))
            return -1;
        if (!reader.skippedChar('?'))
            return -1;
        if (!reader.skippedString(xml_string))
            return -1;
        reader.skipPastSpaces();
        boolean single;
        char qchar;
        if (reader.skippedString(version_string)) {
            reader.skipPastSpaces();
            if (!reader.skippedChar('='))
                return -1;
            reader.skipPastSpaces();
            if (!(single = reader.skippedChar('\'')) && !reader.skippedChar('\"'))
                return -1;
            qchar = single ? '\'' : '\"';
            while (!reader.skippedChar(qchar)) {
                if (!reader.skippedVersionNum())
                    return -1;
            }
            if (!reader.skippedSpace())
                return -1;
            reader.skipPastSpaces();
        }
        else if (xmlDecl)
            return -1;
        if (!reader.skippedString(encoding_string))
            return -1;
        reader.skipPastSpaces();
        if (!reader.skippedChar('='))
            return -1;
        reader.skipPastSpaces();
        if (!(single = reader.skippedChar('\'')) && !reader.skippedChar('\"'))
            return -1;
        int offset = reader.currentOffset();
        qchar = single ? '\'' : '\"';
        if (!reader.skippedAlpha())
            return -1;
        while (!reader.lookingAtChar(qchar)) {
            if (!reader.skippedEncName())
                return -1;
        }
        return reader.addString(offset, reader.currentOffset() - offset);
    }
    //
    // [23] XMLDecl ::= '<?xml' VersionInfo EncodingDecl? SDDecl? S? '?>'
    // [77] TextDecl ::= '<?xml' VersionInfo? EncodingDecl S? '?>'
    //
    private static final char[] xml_string = { 'x','m','l' };
    //
    // [24] VersionInfo ::= S 'version' Eq (' VersionNum ' | " VersionNum ")
    //
    private static final char[] version_string = { 'v','e','r','s','i','o','n' };
    //
    // [80] EncodingDecl ::= S 'encoding' Eq ('"' EncName '"' |  "'" EncName "'" )
    //
    private static final char[] encoding_string = { 'e','n','c','o','d','i','n','g' };
}
