/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 *
 * @version Revision: 45 1.7 src/com/ibm/xml/framework/XMLDocumentHandler.java, parser, xml4j2, xml4j2_0_13 
 */
public interface XMLDocumentHandler {
    /**
     * This method is used to determine which style of interface to use for this handler
     * when passing character data and ignorable whitespace.
     *
     * @returns <code>true</code> if the methods with <code>char</code> array parameters should be used;
     *          <code>false</code> if the methods with a <code>StringPool</code> index should be used.
     * @see #characters
     * @see #ignorableWhitespace
     */
    public boolean sendCharDataAsCharArray();
  
    /**
     * If the there is no version info, encoding info, or standalone info,
     * the corresponding index will be set to -1.
     */
    public void startDocument(int versionIndex, int encodingIndex,
                                   int standAloneIndex)  throws Exception;
    /**
     *
     */
    public void endDocument() throws Exception;
    /**
     *
     */
    public void startElement(int elementNameIndex, int attrListIndex) throws Exception;
    /**
     *
     */
    public void endElement(int elementNameIndex) throws Exception;
    /**
     *
     */
    public void startEntityReference(int entityIndex) throws Exception;
    /**
     *
     */
    public void endEntityReference(int entityIndex) throws Exception;
    /**
     *
     */
    public void characters(int dataIndex, boolean cdataSection) throws Exception;
    /**
     *
     */
    public void ignorableWhitespace(int dataIndex, boolean cdataSection) throws Exception;
    /**
     *
     */
    public void processingInstruction(int targetIndex, int dataIndex) throws Exception;
    /**
     *
     */
    public void comment(int dataIndex) throws Exception;
    /**
     *
     */
    public void characters(char ch[], int start, int length, boolean cdataSection) throws Exception;
    /**
     *
     */
    public void ignorableWhitespace(char ch[], int start, int length, boolean cdataSection) throws Exception;
}
