/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

/**
 * This is the interface used to report errors.
 *
 * All errors are reported using an error code and some optional string
 * index parameters.  See the <code>ErrorCode</code> class for details on
 * the set of errors defined.
 *
 * @author  Glenn Marcy
 * @version Revision: 49 1.5 src/com/ibm/xml/framework/XMLErrorHandler.java, parser, xml4j2, xml4j2_0_13 
 * @see XMLParser
 * @see com.ibm.xml.internal.ErrorCode
 */
public interface XMLErrorHandler {
    /**
     * Report an error.
     *
     * @param errorCode Which error occurred.
     */
    public void error(int errorCode) throws Exception;
    /**
     * Report an error with one parameter string.
     *
     * @param errorCode Which error occurred.
     * @param stringIndex1 Additional information about the specific instance of this error.
     */
    public void error1(int errorCode, int stringIndex1) throws Exception;
    /**
     * Report an error with two parameter strings.
     *
     * @param errorCode Which error occurred.
     * @param stringIndex1 Additional information about the specific instance of this error.
     * @param stringIndex2 Additional information about the specific instance of this error.
     */
    public void error2(int errorCode, int stringIndex1, int stringIndex2) throws Exception;
    /**
     * Report an error with three parameter strings.
     *
     * @param errorCode Which error occurred.
     * @param stringIndex1 Additional information about the specific instance of this error.
     * @param stringIndex2 Additional information about the specific instance of this error.
     * @param stringIndex3 Additional information about the specific instance of this error.
     */
    public void error3(int errorCode, int stringIndex1, int stringIndex2,
                       int stringIndex3) throws Exception;
    /**
     * Report an error with four parameter strings.
     *
     * @param errorCode Which error occurred.
     * @param stringIndex1 Additional information about the specific instance of this error.
     * @param stringIndex2 Additional information about the specific instance of this error.
     * @param stringIndex3 Additional information about the specific instance of this error.
     * @param stringIndex4 Additional information about the specific instance of this error.
     */
    public void error4(int errorCode, int stringIndex1, int stringIndex2,
                       int stringIndex3, int stringIndex4) throws Exception;
}
