/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.framework;

import org.xml.sax.InputSource;
import org.xml.sax.Locator;

/**
 * This is the interface to the scanner.
 *
 * This class also provides the parser with location information through
 * the <code>Locator</code> interface.
 *
 * @author  Glenn Marcy
 * @author  Andy Clark, IBM
 * @version Revision: 55 1.6 src/com/ibm/xml/framework/XMLScanner.java, parser, xml4j2, xml4j2_0_13 
 * @see XMLParser
 * @see InputSource
 * @see Locator
 */
public interface XMLScanner 
    {

    //
    // XMLScanner methods
    //

    /**
     * Reset scanner between calls to scanDocument.
     *
     * @param parserState The state to be used for the next parse.
     * @see ParserState
     */
    public void reset(ParserState parserState);

    /**
     * Process the XML file.
     *
     * The scanner stores the markup information in the pools provided
     * by the parser and calls the handlers provided by the parser.
     *
     * @param source Where to find the XML data.
     * @see InputSource
     */
    public void scanDocument(InputSource source) throws Exception;

    /**
     * This method is temporary until we restructure the interface between
     * DefaultScanner#scanContent and XMLReader#scanContent.
     *
     * @return The ContentSpecType for the current element.
     * @see ElementDecl
     * @see ElementDeclPool#getContentSpecType
     */
    public int getCurrentContentSpecType();

    /**
     * Return a <code>Locator</code> object for the parser to use.
     *
     * @return The locator.
     */
    Locator getLocator();

    /**
     * Stops the scanner in a reasonable place and returns.
     */
    public void stop();

    } // interface XMLScanner
