/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.internal;

import com.ibm.xml.framework.ContentSpecNode;
import com.ibm.xml.framework.StringPool;

/**
 *
 * @version Revision: 87 1.4 src/com/ibm/xml/internal/CMLeaf.java, parser, xml4j2, xml4j2_0_13 
 */
class CMLeaf extends CMNode
{
    // -------------------------------------------------------------------
    //  Constructors
    // -------------------------------------------------------------------
    CMLeaf(int type, int elementIndex, int position) throws CMException
    {
        super(type);

        // Insure that its one of the types we require
        if (type() != ContentSpecNode.NODE_LEAF)
            throw new CMException(ErrorCode.E_VAL_LST);

        // Store the element index and position
        fElementIndex = elementIndex;
        fPosition = position;
    }

    CMLeaf(int type, int elementIndex) throws CMException
    {
        super(type);

        // Insure that its one of the types we require
        if (type() != ContentSpecNode.NODE_LEAF)
            throw new CMException(ErrorCode.E_VAL_LST);

        // Store the element index and position
        fElementIndex = elementIndex;
    }


    // -------------------------------------------------------------------
    //  Package final methods
    // -------------------------------------------------------------------
    final int getElemIndex()
    {
        return fElementIndex;
    }

    final int getPosition()
    {
        return fPosition;
    }

    final void setPosition(int newPosition)
    {
        fPosition = newPosition;
    }


    // -------------------------------------------------------------------
    //  Package, inherited methods
    // -------------------------------------------------------------------
    boolean isNullable() throws CMException
    {
        // Leaf nodes are never nullable unless its an epsilon node
        return (fPosition == -1);
    }

    String toString(StringPool stringPool)
    {
        StringBuffer strRet = new StringBuffer(stringPool.toString(fElementIndex));

        if (fPosition >= 0)
        {
            strRet.append
            (
                " (Pos:"
                + new Integer(fPosition).toString()
                + ")"
            );
        }
        return strRet.toString();
    }


    // -------------------------------------------------------------------
    //  Protected, inherited methods
    // -------------------------------------------------------------------
    protected void calcFirstPos(CMStateSet toSet) throws CMException
    {
        // If we are an epsilon node, then the first pos is an empty set
        if (fPosition == -1)
            toSet.zeroBits();

        // Otherwise, its just the one bit of our position
        else
            toSet.setBit(fPosition);
    }

    protected void calcLastPos(CMStateSet toSet) throws CMException
    {
        // If we are an epsilon node, then the last pos is an empty set
        if (fPosition == -1)
            toSet.zeroBits();

        // Otherwise, its just the one bit of our position
        else
            toSet.setBit(fPosition);
    }


    // -------------------------------------------------------------------
    //  Private data members
    //
    //  fElementIndex
    //      This is the element decl pool index of the element that this
    //      leaf represents.
    //
    //  fPosition
    //      Part of the algorithm to convert a regex directly to a DFA
    //      numbers each leaf sequentially. If its -1, that means its an
    //      epsilon node. Zero and greater are non-epsilon positions.
    // -------------------------------------------------------------------
    private int fElementIndex   = 0;
    private int fPosition       = -1;
};
