/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.internal;

import com.ibm.xml.framework.ContentSpecNode;

/**
 *
 * @version Revision: 89 1.3 src/com/ibm/xml/internal/CMUniOp.java, parser, xml4j2, xml4j2_0_13 
 */
class CMUniOp extends CMNode
{
    // -------------------------------------------------------------------
    //  Constructors
    // -------------------------------------------------------------------
    CMUniOp(int type, CMNode childNode) throws CMException
    {
        super(type);

        // Insure that its one of the types we require
        if ((type() != ContentSpecNode.NODE_ZERO_OR_ONE)
        &&  (type() != ContentSpecNode.NODE_ZERO_OR_MORE)
        &&  (type() != ContentSpecNode.NODE_ONE_OR_MORE))
        {
            throw new CMException(ErrorCode.E_VAL_UST);
        }

        // Store the node and init any data that needs it
        fChild = childNode;
    }


    // -------------------------------------------------------------------
    //  Package, final methods
    // -------------------------------------------------------------------
    final CMNode getChild()
    {
        return fChild;
    }


    // -------------------------------------------------------------------
    //  Package, inherited methods
    // -------------------------------------------------------------------
    boolean isNullable() throws CMException
    {
        //
        //  For debugging purposes, make sure we got rid of all non '*'
        //  repetitions. Otherwise, '*' style nodes are always nullable.
        //
        if ((type() == ContentSpecNode.NODE_ZERO_OR_ONE)
        ||  (type() == ContentSpecNode.NODE_ONE_OR_MORE))
        {
            throw new CMException(ErrorCode.E_VAL_UST);
        }
        return true;
    }


    // -------------------------------------------------------------------
    //  Protected, inherited methods
    // -------------------------------------------------------------------
    protected void calcFirstPos(CMStateSet toSet) throws CMException
    {
        // Its just based on our child node's first pos
        toSet.setTo(fChild.firstPos());
    }

    protected void calcLastPos(CMStateSet toSet) throws CMException
    {
        // Its just based on our child node's last pos
        toSet.setTo(fChild.lastPos());
    }


    // -------------------------------------------------------------------
    //  Private data members
    //
    //  fChild
    //      This is the reference to the one child that we have for this
    //      unary operation.
    // -------------------------------------------------------------------
    private CMNode  fChild;
};
