/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package com.ibm.xml.internal;

import com.ibm.xml.framework.*;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import org.xml.sax.InputSource;

/**
 *
 * @version Revision: 23 1.2 src/com/ibm/xml/internal/EBCDICRecognizer.java, parser, xml4j2, xml4j2_0_13 
 */
public class EBCDICRecognizer extends XMLDeclRecognizer {
    //
    //
    //
    public XMLReader recognize(ParserState parserState, InputSource source,
                               ChunkyByteArray data, boolean xmlDecl) throws Exception
    {
        XMLReader reader = null;
        byte b0 = data.byteAt(0);
        byte b1 = data.byteAt(1);
        byte b2 = data.byteAt(2);
        byte b3 = data.byteAt(3);
        if (b0 != 0x4c || b1 != 0x6f || b2 != (byte)0xa7 || b3 != (byte)0x94)
            return reader;
        XMLReader tempReader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
                                              new InputStreamReader(data, "CP037"));
        int encoding = prescanXMLDeclOrTextDecl(tempReader, xmlDecl);
        if (encoding == -1) {
            data.rewind();
            reader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
                                    new InputStreamReader(data, "UTF8"));
            return reader;
        }
        String enc = parserState.getStringPool().orphanString(encoding).toUpperCase();
        if ("ISO-10646-UCS-2".equals(enc)) throw new UnsupportedEncodingException(enc);
        if ("ISO-10646-UCS-4".equals(enc)) throw new UnsupportedEncodingException(enc);
        if ("UTF-16".equals(enc)) throw new UnsupportedEncodingException(enc);
        String javaencname = MIME2Java.convert(enc);
        if (null == javaencname) {
                                                // Not supported
            if (parserState.getAllowJavaEncodingName()) {
                javaencname = enc;
            } else {
                throw new UnsupportedEncodingException(enc);
            }
        }
        try {
            data.rewind();
            reader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
                                    new InputStreamReader(data, javaencname));
        } catch (UnsupportedEncodingException e) {
            throw e;
        } catch (Exception e) {
            e.printStackTrace();            // Internal Error
        }
        return reader;
    }
}
