/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package com.ibm.xml.internal;

/**
 *
 * @version Revision: 81 1.13 src/com/ibm/xml/internal/ErrorCode.java, parser, xml4j2, xml4j2_0_13 
 */
public final class ErrorCode {
    //
    // Error codes
    //
    public static final int
        E_EXCLAM0 = 0,        // "\"<!--\" or \"<![CDATA[\" expected." (Note: string is "E_!0")
//        E_EXCLAM1 = 1,        // { "\"<!--\" or \"<!DOCTYPE\" expected." (Note: string is "E_!1")
        E_ATTD0 = 2,          // "Attribute name expected."
//        E_ATTD1 = 3,          // "`NOTATION'' is expected."
        E_ATTD2 = 4,          // "Attribute type expected. (e.g. \"CDATA\", \"ID\", \"ENTITY\", etc.)"
        E_ATTD3 = 5,          // "Invalid attribute default declaration. \"#\" must be followed by \"REQUIRED\", \"IMPLIED\", or \"FIXED\"."
        E_ATTD4 = 6,          // "Attribute default declaration expected. (\"#REQUIRED\", \"#IMPLIED\", or \"#FIXED\")"
//        E_ATTD5 = 7,          // "Syntax error in ATTLIST."
        E_ATTEQ0 = 8,         // "''='' expected after attribute name, \"{0}\"."
        E_ATTEQ1 = 9,         // "Duplicate attribute name, \"{0}\"."
        W_ATTEQ2 = 10,        // "Invalid language identifier, \"{0}\", specified for attribute name \"xml:lang\"."
        E_ATTL0 = 11,         // "Element name expected."
//        E_ATTL1 = 12,         // "Syntax error in ATTLIST: `{0}''"
        V_ATTL2 = 13,         // "Duplicate attribute, \"{0}\", specified."
        W_ATTL1 = 14,         // "Warning: attribute \"xml:space\" is recommended to be declared as \"xml:space (default|preserve) #IMPLIED\"."
        W_ATTL2 = 15,         // "Warning: attribute \"{0}\" is already declared."
        E_ATTVAL0 = 16,       // "' or \" expected. Attribute value must be quoted."
        E_ATTVAL1 = 17,       // "Attribute value must not contain '<'."
        E_CDATA0 = 18,        // "\"<!--\" or \"<![CDATA[\" expected."
        E_CDATA1 = 19,        // "\"<![CDATA[\" must end with \"]]>\"."
        E_COM0 = 20,          // "Comment must start with \"<!--\"."
        E_COM1 = 21,          // "Comment must end with \"-->\"."
        E_COM2 = 22,          // "Comment must not contain \"--\"."
        E_COND0 = 23,         // "\"<![INCLUDE[\" or \"<![IGNORE[\" expected."
        E_COND1 = 24,         // "\"<![INCLUDE[\" or \"<![IGNORE[\" must end with \"]]>\"."
        E_COND3 = 25,         // "\"<?\" or \"<!\" expected."
        V_CONT0 = 26,         // "Element \"<{0}>\" is not valid in this context."
        V_CONT1 = 27,         // "Element \"<{0}>\" is not valid because it does not follow the rule, \"{1}\"."
//        E_CS0 = 28,           // "A name is expected."
        E_CS1 = 29,           // "Element content sequence token expected. ('|', ',', or ')')"
        E_CS2 = 30,           // "\"PCDATA\" expected. \"#\" must be followed by \"PCDATA\"."
        E_CS3 = 31,           // "')' expected."
//        E_CS4 = 32,           // "\")*\" is expected."
        E_CS5 = 33,           // "'(' or element name expected."
        E_CS6 = 34,           // "Invalid operator, ''{0}''.  (Previous operator was ''{1}''.)"
//        V_CS7 = 35,           // "A parameter entity in content models must include proper pairs of parenthesises and not be empty and neither the first nor last character should be `,'' or `|'': `{0}''"
//        E_CS8 = 36,           // "`('' is expected: `{0}''"
        E_CS9 = 37,           // "Content model, \"{0}\", must match the mixed model \"(#PCDATA|foo|...|bar)*\"."
        E_CSa = 38,           // "Content model must match the mixed model \"(#PCDATA|foo|...|bar)*\"."
        E_DOCTYPE0 = 39,      // "\"<!--\" or \"<!DOCTYPE\" expected."
        E_DOCTYPE1 = 40,      // "No root element type name."
        E_DOCTYPE2 = 41,      // "Unknown external identifier, \"{0}\". \"SYSTEM\" or \"PUBLIC\" expected."
        E_DOCTYPE3 = 42,      // "DTD is empty."
        E_DTD0 = 43,          // "Invalid character, ''{0}'', in DTD. (Unicode: 0x{1})"
        E_DTD1 = 44,          // "\"<?\" or \"<!\" expected."
        E_DTD2 = 45,          // "Unknown declaration type, \"{0}\". Must be \"ELEMENT\", \"ATTLIST\", \"ENTITY\", or \"NOTATION\"."
        E_DTD3 = 46,          // "Declaration type expected. Must be \"ELEMENT\", \"ATTLIST\", \"ENTITY\", or \"NOTATION\"."
        E_DTD4 = 47,          // "Conditional section only allowed in external subset."
        E_ELEM0 = 48,         // "Element name expected."
        E_ELEM1 = 49,         // "\"EMPTY\", \"ANY\", or '(' expected."
        E_ELEM2 = 50,         // "'>' expected."
        V_ELEM3 = 51,         // "Duplicate element name, \"{0}\"."
        E_ENC0 = 52,          // "Unsupported encoding, \"{0}\"."
        E_ENC1 = 53,          // "Invalid XML character. (Unicode: 0x{0})"
        E_ENC2 = 54,          // "Invalid UTF-16 surrogate. (bytes: 0x{0} 0x{1})"
        E_ENC3 = 55,          // "Invalid UTF-16 code. (byte: 0x{0})"
        E_ENC4 = 56,          // "Invalid UTF-8 code. (byte: 0x{0})"
        E_ENC5 = 57,          // "Invalid UTF-8 code. (bytes: 0x{0} 0x{1})"
        E_ENC6 = 58,          // "Invalid UTF-8 code. (bytes: 0x{0} 0x{1} 0x{2})"
        E_ENC7 = 59,          // "Invalid UTF-8 code. (bytes: 0x{0} 0x{1} 0x{2} 0x{3})"
        E_ENTITY0 = 60,       // "Undefined entity reference, \"&{0};\"."
        E_ENTITY1 = 61,       // "Invalid ENTITY declaration. Name expected."
        E_ENTITY2 = 62,       // "NDATA reference, \"&{0};\", is invalid in this context."
        E_ENTITY3 = 63,       // "Invalid XML character. (Unicode: 0x{0})"
        E_ENTITY4 = 64,       // "'>' expected."
        E_ENTITY5 = 65,       // "Invalid entity declaration. NDATA expected."
        W_ENTITY6 = 66,       // "Warning: Entity name, \"{0}\", already defined. This declaration will be ignored."
        E_ENTITY7 = 67,       // "External entity reference, \"&{0};\", is not allowed here."
        V_ENTITY8 = 68,       // "NDATA type, \"{0}\", is not declared by \"<!NOTATION>\"."
        V_ENTITY9 = 69,       // "External entity, \"&{0};\", is not allowed in standalone document."
        E_ENTITYa = 70,       // "Parameter entity references is not allowed here."
        E_ENUM0 = 71,         // "')' expected."
        E_ENUM1 = 72,         // "Invalid name enumeration, \"(Name | Name | ...)\"."
        E_ENUM2 = 73,         // "Invalid name token enumeration, \"(Nmtoken | Nmtoken | ...)\"."
        E_ENUM3 = 74,         // "'|' or ')' expected."
//        E_ENUM4 = 75,         // "Value `{0}'' is already specified."
        E_ENUM5 = 76,         // "NOTATION, \"{0}\", is not declared."
        E_EOF = 77,           // "Unexpected end of file."
        E_EXT0 = 78,          // "Whitespace expected."
        E_EXT1 = 79,          // "' or \" expected. Public identifier must be quoted."
        E_EXT2 = 80,          // "Invalid character, ''{0}'', in public identifier."
        E_EXT3 = 81,          // "' or \" expected. System identifier must be quoted."
        E_EXT4 = 82,          // "Invalid character, ''{0}'', in system identifier, \"{1}\"."
        E_EXT5 = 83,          // "Space or '>' expected."
        V_IDREF0 = 84,        // "Identifier, \"{0}\", is not defined in the document."
        E_INVCHAR0 = 85,      // "Invalid XML character. (Unicode: 0x{0})"
        E_INVENC0 = 86,       // "Invalid encoding. Big-endian UCS-2 with no byte order mark."
        E_INVENC1 = 87,       // "Invalid encoding. Little-endian UCS-2 with no byte order mark."
        E_IO0 = 88,           // "File, \"{0}\", not found."
        E_NAMES0 = 89,        // "\"{0}\" contains invalid character."
        E_NMTOK0 = 90,        // "\"{0}\" contains invalid character."
        E_NOT0 = 91,          // "Invalid NOTATION declaration. Name expected."
        E_PEREF0 = 92,        // "Parameter entity name expected."
        E_PEREF1 = 93,        // "Parameter reference, \"%{0};\", not terminated with '';''."
        V_PEREF2 = 94,        // "Undefined parameter reference, \"%{0};\"."
//        E_PEREF3 = 95,        // "A parameter reference `%{0};'' is not within single markup declaration: \"{1}\""
        E_PEREF4 = 96,        // "In an internal subset of DTD, parameter entity references within markup declarations are not allowed."
        E_PEREF5 = 97,        // "Recursive reference \"%{0};\". (Reference path: {1})"
//        E_PEREF6 = 98,        // "Parameter-entity replacement text must not be empty in content model: `%{0};''"
        V_PEREF7 = 99,        // "Parameter entity replacement text must include proper pairs of parentheses in content model, \"%{0};\"."
//        E_PEREF8 = 100,       // "Neither the first nor last character in parameter-entity replacement text must be `,'' or `|'': `%{0};''"
        E_PEREF9 = 101,       // "Parameter entity replacement text must include declarations or proper pairs of ''<'' and ''>''. (entity: \"%{0};\")"
        E_PI0 = 102,          // "Processing instruction name expected."
//        E_PI1 = 103,          // "No space after PI target name."
        E_PI2 = 104,          // "Processing instruction data expected."
        E_PI3 = 105,          // "\"?>\" expected."
        E_PI4 = 106,          // "Invalid encoding name, \"{0}\"."
        E_PI5 = 107,          // "A processing instruction target matching \"[xX][mM][lL]\" is not allowed."
        E_PI6 = 108,          // "Invalid parameter, \"{0}\", in processing instruction declaration."
        E_PI7 = 109,          // "No \"encoding\" parameter specified."
        E_REFER0 = 110,       // "Invalid character, ''{0}'', in reference."
        E_REFER1 = 111,       // "Reference must end with ';'."
        E_REFER2 = 112,       // "Number expected."
        E_REFER3 = 113,       // "Character reference, \"&#x{0}\", is out of range."
        E_REFER4 = 114,       // "Character reference, \"&#{0};\", is out of range."
        E_SPACE = 115,        // "Whitespace expected."
        E_STRUCT0 = 116,      // "Invalid document structure."
        E_STRUCT1 = 117,      // "Document must start with \"<?xml\"."
        E_STRUCT2 = 118,      // "Document must contain root element."
        E_STRUCT3 = 119,      // "DOCTYPE must be placed before root element."
        W_STRUCT4 = 120,      // "Warning: No \"<?xml version=\"1.0\"?>\" found."
        W_STRUCT5 = 121,      // "Warning: No \"<!DOCTYPE ...>\" found."
        E_TAG0 = 122,         // "Element name expected."
        E_TAG1 = 123,         // "\"/>\" or '>' expected."
        E_TAG3 = 124,         // "\"</{0}>\" expected."
        E_TAG4 = 125,         // "'>' expected."
        V_TAG5 = 126,         // "Document root element, \"{1}\", must match DOCTYPE root, \"{0}\"."
        V_TAG6 = 127,         // "Attribute, \"{0}\", is not declared in element, \"{1}\"."
        V_TAG7 = 128,         // "Invalid character, ''{0}'', in attribute value."
        V_TAG8 = 129,         // "Duplicate identifier, \"{0}\"."
        V_TAG9 = 130,         // "Invalid attribute value, \"{0}\"."
        V_TAGa = 131,         // "Attribute value, \"{0}\", is not binary external entity."
        V_TAGb = 132,         // "Invalid attribute value, \"{0}\". No NOTATION declared for \"{0}\"."
        V_TAGc = 133,         // "Required attribute, \"{0}\", is not specified."
        V_TAGd = 134,         // "Invalid value, \"{2}\", specified for attribute, \"{0}\". (default value: \"{1}\")"
        E_TAGe = 135,         // "\"{0}\" may only have one name space separator, '':''."
        E_TAGf = 136,         // "Name space expected in \"{0}\"."
        E_TAGg = 137,         // "Unexpected end tag."
        E_TAGh = 138,         // "General entity can only have entire elements, references, CDATA, processing instructions, and text."
        E_TAGi = 139,         // "Recursive reference, \"&{0};\". (reference path: {1})"
        E_TAGj = 140,         // "Text must not contain \"]]>\"."
        E_TAGk = 141,         // "Element `{0}'' is declared as `EMPTY''.  `/>'' is expected."
        E_TAGl = 142,         // "Attribute, \"{0}\", value expected."
        E_TAGn = 143,         // "Attribute `{0}'' (`{1}'' as namespace, `{2}'' as localPart) is already specified as `{3}'' in the same tag."
        E_TAGm = 144,         // "Attribute \"{0}\" is already specified as \"{3}\". (namespace: \"{1}\", local: \"{2}\")"
        E_XML0 = 145,         // "\"?>\" expected."
        E_XML1 = 146,         // "Attribute, \"version\", is required."
        E_XML2 = 147,         // "Attribute, \"version\", must be specified first."
        E_XML3 = 148,         // "Invalid \"standalone\" attribute value, \"{0}\". Must be \"yes\" or \"no\"."
        E_XML4 = 149,         // "Unsupported XML version."
        E_XML5 = 150,         // "Unknown attribute, \"{0}\", or invalid attribute order."
        E_XML6 = 151,         // "Invalid XML version format, \"{0}\"."
        //
        // XML4J2 parser additions
        //
        E_VAL_CST = 152,      // "Unknown ContentSpecNode.NODE_XXX value"
        E_VAL_UST = 153,      // "Invalid ContentSpecNode.NODE_XXX value for unary op CMNode"
        E_VAL_BST = 154,      // "Invalid ContentSpecNode.NODE_XXX value for binary op CMNode"
        E_VAL_LST = 155,      // "Invalid ContentSpecNode.NODE_XXX value for leaf CMNode"
        E_VAL_CMSI = 156,     // "Invalid CMStateSet bit index"
        E_VAL_NIICM = 157,    // "Only * unary ops should be in the internal content model tree"
        E_VAL_WCGHI = 158,    // "Input to whatCanGoHere() is inconsistent"
        E_VAL_NPCD = 159,     // "PCData found in non-mixed model content"
        E_VAL_NRE = 160,      // "There is no root element set"
        //
        V_TAGo = 161,         // "Invalid attribute value, \"{0}\". Must be one of \"{1}\"."
        E_PEREFa = 162,       // "External entity reference, \"%{0};\", is not allowed here."
        E_NOT1 = 163,         // "'>' expected."
        //
        V_CONT2 = 164,        // "Element \"<{0}>\" contains incomplete content for the rule, \"{1}\"." },
        W_DTD5 = 165,         // "Element, \"{0}\", refers to undeclared element, \"{1}\", in content model" },
        V_ELEM4 = 166,        // "Element, \"{0}\", is undeclared" },
		E_INT_DCN = 167,      // "Internal Error: dataChunk == NULL"
		E_INT_PCN = 168,      // "Internal Error: fPreviousChunk == NULL"
		E_INT_ATR = 169,      // "Internal Error in AttrPool: unable to reclaim Attlist"
		E_INT_ISS = 170,      // "Internal Error in DefaultEntityHandler: InputSource stack out-of-sync"
		E_INT_REVAL = 171,    // "Cannot revalidate a non-element node"
		E_INT_MSGFMT = 172,   // "Internal Error while formatting error message"
		E_ATTD5 = 173,        // "Invalid default value for attribute declaration"

		//
        // KEEP THIS UP TO DATE !!
        //
        E_MAX_ERROR = 173;
    //
    // Table of ErrorCode to error key strings.
    // Note: We keep the spaces reserved for old unused errors from XML4J.
    //
    private static final String[] fgErrorKeys = {
        "E_!0",       //   0, "\"<!--\" or \"<![CDATA[\" expected." (Note: ErrorCode is E_EXCLAM0)
        "E_!1",       //   1, "\"<!--\" or \"<!DOCTYPE\" expected." (Note: ErrorCode is E_EXCLAM1)
        "E_ATTD0",    //   2, "Attribute name expected."
        "E_ATTD1",    //   3, "`NOTATION'' is expected."
        "E_ATTD2",    //   4, "Attribute type expected. (e.g. \"CDATA\", \"ID\", \"ENTITY\", etc.)"
        "E_ATTD3",    //   5, "Invalid attribute default declaration. \"#\" must be followed by \"REQUIRED\", \"IMPLIED\", or \"FIXED\"."
        "E_ATTD4",    //   6, "Attribute default declaration expected. (\"#REQUIRED\", \"#IMPLIED\", or \"#FIXED\")"
        "E_ATTD5",    //   7, "Syntax error in ATTLIST."
        "E_ATTEQ0",   //   8, "''='' expected after attribute name, \"{0}\"."
        "E_ATTEQ1",   //   9, "Duplicate attribute name, \"{0}\"."
        "W_ATTEQ2",   //  10, "Invalid language identifier, \"{0}\", specified for attribute name \"xml:lang\"."
        "E_ATTL0",    //  11, "Element name expected."
        "E_ATTL1",    //  12, "Syntax error in ATTLIST: `{0}''"
        "V_ATTL2",    //  13, "Duplicate attribute, \"{0}\", specified."
        "W_ATTL1",    //  14, "Warning: attribute \"xml:space\" is recommended to be declared as \"xml:space (default|preserve) #IMPLIED\"."
        "W_ATTL2",    //  15, "Warning: attribute \"{0}\" is already declared."
        "E_ATTVAL0",  //  16, "' or \" expected. Attribute value must be quoted."
        "E_ATTVAL1",  //  17, "Attribute value must not contain '<'."
        "E_CDATA0",   //  18, "\"<!--\" or \"<![CDATA[\" expected."
        "E_CDATA1",   //  19, "\"<![CDATA[\" must end with \"]]>\"."
        "E_COM0",     //  20, "Comment must start with \"<!--\"."
        "E_COM1",     //  21, "Comment must end with \"-->\"."
        "E_COM2",     //  22, "Comment must not contain \"--\"."
        "E_COND0",    //  23, "\"<![INCLUDE[\" or \"<![IGNORE[\" expected."
        "E_COND1",    //  24, "\"<![INCLUDE[\" or \"<![IGNORE[\" must end with \"]]>\"."
        "E_COND3",    //  25, "\"<?\" or \"<!\" expected."
        "V_CONT0",    //  26, "Element \"<{0}>\" is not valid in this context."
        "V_CONT1",    //  27, "Element \"<{0}>\" is not valid because it does not follow the rule, \"{1}\"."
        "E_CS0",      //  28, "A name is expected."
        "E_CS1",      //  29, "Element content sequence token expected. ('|', ',', or ')')"
        "E_CS2",      //  30, "\"PCDATA\" expected. \"#\" must be followed by \"PCDATA\"."
        "E_CS3",      //  31, "')' expected."
        "E_CS4",      //  32, "\")*\" is expected."
        "E_CS5",      //  33, "'(' or element name expected."
        "E_CS6",      //  34, "Invalid operator, ''{0}''.  (Previous operator was ''{1}''.)"
        "V_CS7",      //  35, "A parameter entity in content models must include proper pairs of parenthesises and not be empty and neither the first nor last character should be `,'' or `|'': `{0}''"
        "E_CS8",      //  36, "`('' is expected: `{0}''"
        "E_CS9",      //  37, "Content model, \"{0}\", must match the mixed model \"(#PCDATA|foo|...|bar)*\"."
        "E_CSa",      //  38, "Content model must match the mixed model \"(#PCDATA|foo|...|bar)*\"."
        "E_DOCTYPE0", //  39, "\"<!--\" or \"<!DOCTYPE\" expected."
        "E_DOCTYPE1", //  40, "No root element type name."
        "E_DOCTYPE2", //  41, "Unknown external identifier, \"{0}\". \"SYSTEM\" or \"PUBLIC\" expected."
        "E_DOCTYPE3", //  42, "DTD is empty."
        "E_DTD0",     //  43, "Invalid character, ''{0}'', in DTD. (Unicode: 0x{1})"
        "E_DTD1",     //  44, "\"<?\" or \"<!\" expected."
        "E_DTD2",     //  45, "Unknown declaration type, \"{0}\". Must be \"ELEMENT\", \"ATTLIST\", \"ENTITY\", or \"NOTATION\"."
        "E_DTD3",     //  46, "Declaration type expected. Must be \"ELEMENT\", \"ATTLIST\", \"ENTITY\", or \"NOTATION\"."
        "E_DTD4",     //  47, "Conditional section only allowed in external subset."
        "E_ELEM0",    //  48, "Element name expected."
        "E_ELEM1",    //  49, "\"EMPTY\", \"ANY\", or '(' expected."
        "E_ELEM2",    //  50, "'>' expected."
        "V_ELEM3",    //  51, "Duplicate element name, \"{0}\"."
        "E_ENC0",     //  52, "Unsupported encoding, \"{0}\"."
        "E_ENC1",     //  53, "Invalid XML character. (Unicode: 0x{0})"
        "E_ENC2",     //  54, "Invalid UTF-16 surrogate. (bytes: 0x{0} 0x{1})"
        "E_ENC3",     //  55, "Invalid UTF-16 code. (byte: 0x{0})"
        "E_ENC4",     //  56, "Invalid UTF-8 code. (byte: 0x{0})"
        "E_ENC5",     //  57, "Invalid UTF-8 code. (bytes: 0x{0} 0x{1})"
        "E_ENC6",     //  58, "Invalid UTF-8 code. (bytes: 0x{0} 0x{1} 0x{2})"
        "E_ENC7",     //  59, "Invalid UTF-8 code. (bytes: 0x{0} 0x{1} 0x{2} 0x{3})"
        "E_ENTITY0",  //  60, "Undefined entity reference, \"&{0};\"."
        "E_ENTITY1",  //  61, "Invalid ENTITY declaration. Name expected."
        "E_ENTITY2",  //  62, "NDATA reference, \"&{0};\", is invalid in this context."
        "E_ENTITY3",  //  63, "Invalid XML character. (Unicode: 0x{0})"
        "E_ENTITY4",  //  64, "'>' expected."
        "E_ENTITY5",  //  65, "Invalid entity declaration. NDATA expected."
        "W_ENTITY6",  //  66, "Warning: Entity name, \"{0}\", already defined. This declaration will be ignored."
        "E_ENTITY7",  //  67, "External entity reference, \"&{0};\", is not allowed here."
        "V_ENTITY8",  //  68, "NDATA type, \"{0}\", is not declared by \"<!NOTATION>\"."
        "V_ENTITY9",  //  69, "External entity, \"&{0};\", is not allowed in standalone document."
        "E_ENTITYa",  //  70, "Parameter entity references is not allowed here."
        "E_ENUM0",    //  71, "')' expected."
        "E_ENUM1",    //  72, "Invalid name enumeration, \"(Name | Name | ...)\"."
        "E_ENUM2",    //  73, "Invalid name token enumeration, \"(Nmtoken | Nmtoken | ...)\"."
        "E_ENUM3",    //  74, "'|' or ')' expected."
        "E_ENUM4",    //  75, "Value `{0}'' is already specified."
        "E_ENUM5",    //  76, "NOTATION, \"{0}\", is not declared."
        "E_EOF",      //  77, "Unexpected end of file."
        "E_EXT0",     //  78, "Whitespace expected."
        "E_EXT1",     //  79, "' or \" expected. Public identifier must be quoted."
        "E_EXT2",     //  80, "Invalid character, ''{0}'', in public identifier."
        "E_EXT3",     //  81, "' or \" expected. System identifier must be quoted."
        "E_EXT4",     //  82, "Invalid character, ''{0}'', in system identifier, \"{1}\"."
        "E_EXT5",     //  83, "Space or '>' expected."
        "V_IDREF0",   //  84, "Identifier, \"{0}\", is not defined in the document."
        "E_INVCHAR0", //  85, "Invalid XML character. (Unicode: 0x{0})"
        "E_INVENC0",  //  86, "Invalid encoding. Big-endian UCS-2 with no byte order mark."
        "E_INVENC1",  //  87, "Invalid encoding. Little-endian UCS-2 with no byte order mark."
        "E_IO0",      //  88, "File, \"{0}\", not found."
        "E_NAMES0",   //  89, "\"{0}\" contains invalid character."
        "E_NMTOK0",   //  90, "\"{0}\" contains invalid character."
        "E_NOT0",     //  91, "Invalid NOTATION declaration. Name expected."
        "E_PEREF0",   //  92, "Parameter entity name expected."
        "E_PEREF1",   //  93, "Parameter reference, \"%{0};\", not terminated with '';''."
        "V_PEREF2",   //  94, "Undefined parameter reference, \"%{0};\"."
        "E_PEREF3",   //  95, "A parameter reference `%{0};'' is not within single markup declaration: \"{1}\""
        "E_PEREF4",   //  96, "In an internal subset of DTD, parameter entity references within markup declarations are not allowed."
        "E_PEREF5",   //  97, "Recursive reference \"%{0};\". (Reference path: {1})"
        "E_PEREF6",   //  98, "Parameter-entity replacement text must not be empty in content model: `%{0};''"
        "V_PEREF7",   //  99, "Parameter entity replacement text must include proper pairs of parentheses in content model, \"%{0};\"."
        "E_PEREF8",   // 100, "Neither the first nor last character in parameter-entity replacement text must be `,'' or `|'': `%{0};''"
        "E_PEREF9",   // 101, "Parameter entity replacement text must include declarations or proper pairs of ''<'' and ''>''. (entity: \"%{0};\")"
        "E_PI0",      // 102, "Processing instruction name expected."
        "E_PI1",      // 103, "No space after PI target name."
        "E_PI2",      // 104, "Processing instruction data expected."
        "E_PI3",      // 105, "\"?>\" expected."
        "E_PI4",      // 106, "Invalid encoding name, \"{0}\"."
        "E_PI5",      // 107, "A processing instruction target matching \"[xX][mM][lL]\" is not allowed."
        "E_PI6",      // 108, "Invalid parameter, \"{0}\", in processing instruction declaration."
        "E_PI7",      // 109, "No \"encoding\" parameter specified."
        "E_REFER0",   // 110, "Invalid character, ''{0}'', in reference."
        "E_REFER1",   // 111, "Reference must end with ';'."
        "E_REFER2",   // 112, "Number expected."
        "E_REFER3",   // 113, "Character reference, \"&#x{0}\", is out of range."
        "E_REFER4",   // 114, "Character reference, \"&#{0};\", is out of range."
        "E_SPACE",    // 115, "Whitespace expected."
        "E_STRUCT0",  // 116, "Invalid document structure."
        "E_STRUCT1",  // 117, "Document must start with \"<?xml\"."
        "E_STRUCT2",  // 118, "Document must contain root element."
        "E_STRUCT3",  // 119, "DOCTYPE must be placed before root element."
        "W_STRUCT4",  // 120, "Warning: No \"<?xml version=\"1.0\"?>\" found."
        "W_STRUCT5",  // 121, "Warning: No \"<!DOCTYPE ...>\" found."
        "E_TAG0",     // 122, "Element name expected."
        "E_TAG1",     // 123, "\"/>\" or '>' expected."
        "E_TAG3",     // 124, "\"</{0}>\" expected."
        "E_TAG4",     // 125, "'>' expected."
        "V_TAG5",     // 126, "Document root element, \"{1}\", must match DOCTYPE root, \"{0}\"."
        "V_TAG6",     // 127, "Attribute, \"{0}\", is not declared in element, \"{1}\"."
        "V_TAG7",     // 128, "Invalid character, ''{0}'', in attribute value."
        "V_TAG8",     // 129, "Duplicate identifier, \"{0}\"."
        "V_TAG9",     // 130, "Invalid attribute value, \"{0}\"."
        "V_TAGa",     // 131, "Attribute value, \"{0}\", is not binary external entity."
        "V_TAGb",     // 132, "Invalid attribute value, \"{0}\". No NOTATION declared for \"{0}\"."
        "V_TAGc",     // 133, "Required attribute, \"{0}\", is not specified."
        "V_TAGd",     // 134, "Invalid value, \"{2}\", specified for attribute, \"{0}\". (default value: \"{1}\")"
        "E_TAGe",     // 135, "\"{0}\" may only have one name space separator, '':''."
        "E_TAGf",     // 136, "Name space expected in \"{0}\"."
        "E_TAGg",     // 137, "Unexpected end tag."
        "E_TAGh",     // 138, "General entity can only have entire elements, references, CDATA, processing instructions, and text."
        "E_TAGi",     // 139, "Recursive reference, \"&{0};\". (reference path: {1})"
        "E_TAGj",     // 140, "Text must not contain \"]]>\"."
        "E_TAGk",     // 141, "Element `{0}'' is declared as `EMPTY''.  `/>'' is expected."
        "E_TAGl",     // 142, "Attribute, \"{0}\", value expected."
        "E_TAGn",     // 143, "Attribute `{0}'' (`{1}'' as namespace, `{2}'' as localPart) is already specified as `{3}'' in the same tag."
        "E_TAGm",     // 144, "Attribute \"{0}\" is already specified as \"{3}\". (namespace: \"{1}\", local: \"{2}\")"
        "E_XML0",     // 145, "\"?>\" expected."
        "E_XML1",     // 146, "Attribute, \"version\", is required."
        "E_XML2",     // 147, "Attribute, \"version\", must be specified first."
        "E_XML3",     // 148, "Invalid \"standalone\" attribute value, \"{0}\". Must be \"yes\" or \"no\"."
        "E_XML4",     // 149, "Unsupported XML version."
        "E_XML5",     // 150, "Unknown attribute, \"{0}\", or invalid attribute order."
        "E_XML6",     // 151, "Invalid XML version format, \"{0}\"."
        //
        // XML4J2 additions
        //
        "E_VAL_CST",  // 152, "Unknown ContentSpecNode.NODE_XXX value"
        "E_VAL_UST",  // 153, "Invalid ContentSpecNode.NODE_XXX value for unary op CMNode"
        "E_VAL_BST",  // 154, "Invalid ContentSpecNode.NODE_XXX value for binary op CMNode"
        "E_VAL_LST",  // 155, "Invalid ContentSpecNode.NODE_XXX value for leaf CMNode"
        "E_VAL_CMSI", // 156, "Invalid CMStateSet bit index"
        "E_VAL_NIICM",// 157, "Only * unary ops should be in the internal content model tree"
        "E_VAL_WCGHI",// 158, "Input to whatCanGoHere() is inconsistent"
        "E_VAL_NPCD", // 159, "PCData found in non-mixed model content"
        "E_VAL_NRE",  // 160, "There is no root element set"
        "V_TAGo",     // 161, "Invalid attribute value, \"{0}\". Must be one of \"{1}\"."
        "E_PEREFa",   // 162, "External entity reference, \"%{0};\", is not allowed here."
        "E_NOT1",     // 163, "'>' expected."
        "V_CONT2",    // 164, "Element \"<{0}>\" contains incomplete content for the rule, \"{1}\"." },
        "W_DTD5",     // 165, "Element, \"{0}\", refers to undeclared element, \"{1}\", in content model" },
        "V_ELEM4",    // 166, "Element, \"{0}\", is undeclared" },
		"E_INT_DCN",  // 167, "Internal Error: dataChunk == NULL"
   		"E_INT_PCN",  // 168, "Internal Error: fPreviousChunk == NULL"	
		"E_INT_ATR",  // 169, "Internal Error in AttrPool: unable to reclaim Attlist"
		"E_INT_ISS",  // 170, "Internal Error in DefaultEntityHandler: InputSource stack out-of-sync"  
		"E_INT_REVAL", // 171, "Cannot revalidate a non-element node"
		"E_INT_MSGFMT",// 172, "Internal Error while formatting error message"
		"E_ATTD5",    // 173  "Invalid default value for attribute declaration"

		//
        // KEEP THIS UP TO DATE !!
        //
        null
    };
    //
    //
    //
    public static final String getErrorKey(int errorCode) {
        if (errorCode < 0 || errorCode > fgErrorKeys.length)
            return null;
        return fgErrorKeys[errorCode];
    }
}
