/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package com.ibm.xml.internal;

/**
 * Algorithm used to hash char arrays (strings).
 * The trick seems to be the shift of the top eight bits of the hashcode
 * back down to the bottom to keep them from being rolled out.
 *
 * @version Revision: 66 1.3 src/com/ibm/xml/internal/StringHasher.java, parser, xml4j2, xml4j2_0_13 
 */
public final class StringHasher {
    //
    //
    //
    public static int hashString(String str, int strLength) {
        int hashcode = 0;
        for (int i = 0; i < strLength; i++) {
            int top = hashcode >> 24;
            hashcode += ((hashcode * 37) + top + ((int)str.charAt(i)));
        }
        return finishHash(hashcode);
    }
    //
    //
    //
    public static int hashChar(int hashcode, int position, int ch) {
        int top = hashcode >> 24;
        hashcode += ((hashcode * 37) + top + ch);
        return hashcode;
    }
    //
    //
    //
    public static int finishHash(int hashcode) {
        hashcode = (hashcode & 0x7fffffff);
        return (hashcode == 0) ? 1 : hashcode;
    }
}
