/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package com.ibm.xml.internal;

import com.ibm.xml.framework.*;

import java.io.IOException;
import org.xml.sax.InputSource;

/**
 *
 * @version Revision: 85 1.4 src/com/ibm/xml/internal/UCSRecognizer.java, parser, xml4j2, xml4j2_0_13 
 */
public class UCSRecognizer extends XMLDeclRecognizer {
    //
    //
    //
    public XMLReader recognize(ParserState parserState, InputSource source,
                               ChunkyByteArray data, boolean xmlDecl) throws IOException
    {
        XMLReader reader = null;
        byte b0 = data.byteAt(0);
        if (b0 == 0) {
            int b1 = data.byteAt(1);
            if (b1 == 0) {
                if (data.byteAt(2) == 0 && data.byteAt(3) == '<')
                    reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS4B);
            } else if (b1 == '<') {
                if (data.byteAt(2) == 0 && data.byteAt(3) == '?')
                    reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS2B_NOBOM);
            }
        } else if (b0 == '<') {
            int b1 = data.byteAt(1);
            if (b1 == 0) {
                int b2 = data.byteAt(2);
                if (data.byteAt(3) == 0) {
                    if (b2 == 0)
                        reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS4L);
                    else if (b2 == '?')
                        reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS2L_NOBOM);
                }
            }
        } else if (b0 == (byte)0xfe) {
            if (data.byteAt(1) == (byte)0xff)
                reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS2B);
        } else if (b0 == (byte)0xff) {
            if (data.byteAt(1) == (byte)0xfe)
                reader = new UCSReader(parserState, source.getPublicId(), source.getSystemId(), data, UCSReader.E_UCS2L);
        }
        return reader;
    }
}
