/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package com.ibm.xml.internal;

import com.ibm.xml.framework.*;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import org.xml.sax.InputSource;

/**
 *
 * @version Revision: 84 1.5 src/com/ibm/xml/internal/UTF8Recognizer.java, parser, xml4j2, xml4j2_0_13 
 */
public class UTF8Recognizer extends XMLDeclRecognizer {
    //
    //
    //
    public XMLReader recognize(ParserState parserState, InputSource source,
                               ChunkyByteArray data, boolean xmlDecl) throws Exception {
        XMLReader reader = null;
        byte b0 = data.byteAt(0);
        if (b0 == '<') {
            int b1 = data.byteAt(1);
            if (b1 == '?') {
                if (data.byteAt(2) == 'x' && data.byteAt(3) == 'm' && data.byteAt(4) == 'l') {
                    int b5 = data.byteAt(5);
                    if (b5 == 0x20 || b5 == 0x09 || b5 == 0x0a || b5 == 0x0d) {
                        int encoding = prescanXMLDeclOrTextDecl(new XMLDeclReader(data, parserState), xmlDecl);
                        if (encoding != -1) {
                            String enc = parserState.getStringPool().orphanString(encoding).toUpperCase();
                            if ("ISO-10646-UCS-2".equals(enc)) throw new UnsupportedEncodingException(enc);
                            if ("ISO-10646-UCS-4".equals(enc)) throw new UnsupportedEncodingException(enc);
                            if ("UTF-16".equals(enc)) throw new UnsupportedEncodingException(enc);

                            String javaencname = MIME2Java.convert(enc);
                            if (null == javaencname) {
                                                                    // Not supported
                                if (parserState.getAllowJavaEncodingName()) {
                                    javaencname = enc;
                                } else {
                                    throw new UnsupportedEncodingException(enc);
                                }
                            }
                            try {
                                data.rewind();
                                if ("UTF-8".equalsIgnoreCase(javaencname) || "UTF8".equalsIgnoreCase(javaencname)) {
                                    if (parserState.getUseCharReaderForUTF8())
//                                        reader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
//                                                                new InputStreamReader(data, javaencname));
                                        reader = new UTF8CharReader(parserState, source.getPublicId(), source.getSystemId(), data);
                                    else
                                        reader = new UTF8Reader(parserState, source.getPublicId(), source.getSystemId(), data);
                                } else {
                                    reader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
                                                            new InputStreamReader(data, javaencname));
                                }
                            } catch (UnsupportedEncodingException e) {
                                throw e;
                            } catch (Exception e) {
                                e.printStackTrace();            // Internal Error
                            }
                        } else {
                            data.rewind();
                            if (parserState.getUseCharReaderForUTF8())
//                                reader = new CharReader(parserState, source.getPublicId(), source.getSystemId(),
//                                                        new InputStreamReader(data, "UTF8"));
                                reader = new UTF8CharReader(parserState, source.getPublicId(), source.getSystemId(), data);
                            else
                                reader = new UTF8Reader(parserState, source.getPublicId(), source.getSystemId(), data);
                        }
                    }
                }
            }
        }
        return reader;
    }
}

final class XMLDeclReader extends XMLReader {
    //
    //
    //
    private ChunkyByteArray fData = null;
    private ParserState fParserState;
    //
    //
    //
    XMLDeclReader(ChunkyByteArray data, ParserState state) {
        super(null, null, null);
        fData = data;
        fParserState = state;
    }
    //
    // These methods are used to parse XMLDecl/TextDecl.
    //
    public boolean skippedChar(char ch) throws IOException {
        if (fData.byteAt(fCurrentOffset) != ch)
            return false;
        fCurrentOffset++;
        return true;
    }
    public boolean lookingAtChar(char ch) throws IOException {
        return fData.byteAt(fCurrentOffset) == ch;
    }
    public boolean skippedSpace() throws IOException {
        int ch = fData.byteAt(fCurrentOffset) & 0xff;
        if (ch != 0x20 && ch != 0x09 && ch != 0x0A && ch != 0x0D)
            return false;
        fCurrentOffset++;
        return true;
    }
    public int skipPastSpaces() throws IOException {
        while (true) {
            int ch = fData.byteAt(fCurrentOffset) & 0xff;
            if (ch != 0x20 && ch != 0x09 && ch != 0x0A && ch != 0x0D)
                return fCurrentOffset;
            fCurrentOffset++;
        }
    }
    public boolean skippedAlpha() throws IOException {
        int ch = fData.byteAt(fCurrentOffset) & 0xff;
        if (ch > 'z' || fgAsciiAlphaChar[ch] == 0)
            return false;
        fCurrentOffset++;
        return true;
    }
    private boolean skippedAsciiCharWithFlag(byte flag) throws IOException {
        int ch = fData.byteAt(fCurrentOffset) & 0xff;
        if ((ch & 0x80) != 0 || (fgCharFlags[ch] & flag) == 0)
            return false;
        fCurrentOffset++;
        return true;
    }
    public boolean skippedVersionNum() throws IOException {
        return skippedAsciiCharWithFlag(E_VersionNumFlag);
    }
    public boolean skippedEncName() throws IOException {
        return skippedAsciiCharWithFlag(E_EncNameFlag);
    }
    public boolean skippedString(char[] s) throws IOException {
        int offset = fCurrentOffset;
        for (int i = 0; i < s.length; i++) {
            if (fData.byteAt(offset) != s[i])
                return false;
            offset++;
        }
        fCurrentOffset = offset;
        return true;
    }
    public int addString(int offset, int length) {
        StringBuffer str = new StringBuffer();
        for (int i = 0; i < length; i++) {
            str.append((char)fData.byteAt(offset + i));
            }
        return fParserState.getStringPool().addString(str.toString());
        }
    //
    // The rest of the methods in XMLReader are not used for parsing XMLDecl/TextDecl.
    //
    public int addSymbol(int offset, int length) {
        return -1;
    }
    public void append(ChunkyCharArray charArray, int offset, int length) {
    }
    public int skipOneChar() throws IOException {
        throw new IOException();
    }
    public int skipAsciiChar() throws IOException {
        throw new IOException();
    }
    public int skipToChar(char ch) throws IOException {
        throw new IOException();
    }
    public int skipPastChar(char ch) throws IOException {
        throw new IOException();
    }
    public boolean skippedValidChar() throws IOException {
        throw new IOException();
    }
    public boolean lookingAtValidChar() throws IOException {
        throw new IOException();
    }
    public int skipInvalidChar(int errorCode) throws Exception {
        throw new IOException();
    }
    public boolean lookingAtSpace() throws IOException {
        throw new IOException();
    }
    public int skipDecimalDigit() throws IOException {
        throw new IOException();
    }
    public int skipHexDigit() throws IOException {
        throw new IOException();
    }
    public boolean skippedPubidChar() throws IOException {
        throw new IOException();
    }
    public int scanName(char fastcheck, int expectedName) throws IOException {
        throw new IOException();
    }
    public int skipPastName(char fastcheck) throws IOException {
        throw new IOException();
    }
    public int skipPastNmtoken(char fastcheck) throws IOException {
        throw new IOException();
    }
    public int scanContent(ScanContentState scanState) throws Exception {
        throw new IOException();
    }
}
