/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.parsers;

import com.ibm.xml.framework.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;

/**
 *
 * @version Revision: 64 1.9 src/com/ibm/xml/parsers/NonValidatingParser.java, parser, xml4j2, xml4j2_0_13 
 */
public class NonValidatingParser extends XMLParser
{
    public NonValidatingParser() {
        getParserState().useDefaultStringPool();
        getParserState().useDefaultAttrPool();
        getParserState().useDefaultEntityPool();
        getParserState().useDefaultElementDeclPool();
    }

    //
    // XMLDocumentTypeHandler interface
    //

    /**
     *  This function will be called when a &lt;!DOCTYPE...&gt; declaration is
     *  encountered.
     */
    public void doctypeDecl(int rootElementNameIndex) throws Exception {}

    /**
     *  This function will be called at the start of the "internal subset" of a 
     *  doctype declaration.
     */
    public void startInternalSubset() throws Exception {}

    /**
     *  This function will be called at the end of the "internal subset" of a 
     *  doctype declaration.
     */
    public void endInternalSubset() throws Exception {}

    /**
     *  This function will be called at the start of the "external subset" of a 
     *  doctype declaration.
     */
    public void startExternalSubset(int publicIdStringIndex, int systemIdStringIndex) throws Exception {}
    
    /**
     *  This function will be called at the end of the "external subset" of a 
     *  doctype declaration.
     */
    public void endExternalSubset() throws Exception {}
 
    /**
     *
     */
    public void elementDecl(int elementIndex) throws Exception {
    }
    /**
     *
     */
    public void attlistDecl(int elementIndex, int attDefIndex) throws Exception {
    }
    /**
     *
     */
    public void internalEntityDecl(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void externalEntityDecl(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void unparsedEntityDecl(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void notationDecl(int notationIndex) throws Exception {
    }

    //
    // XMLDocumentHandler interface
    //

    /**
     *
     */
    public void startDocument(int versionIndex,
                              int encodingIndex,
                              int standAloneIndex) throws Exception {
        StringPool  strPool = getParserState().getStringPool();
        strPool.orphanString(versionIndex);
        strPool.orphanString(encodingIndex);
        strPool.orphanString(standAloneIndex);
    }
    /**
     *
     */
    public void endDocument() throws Exception {
    }
    /**
     *
     */
    public void startElement(int elementNameIndex, int attrListIndex) throws Exception {
        getParserState().getAttrPool().releaseAttrList(attrListIndex);
    }
    /**
     *
     */
    public void endElement(int elementNameIndex) throws Exception {
    }
    /**
     *
     */
    public void startEntityReference(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void endEntityReference(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void characters(int dataIndex, boolean cdataSection) throws Exception {
        getParserState().getStringPool().releaseString(dataIndex);
    }
    /**
     *
     */
    public void ignorableWhitespace(int dataIndex, boolean cdataSection) throws Exception {
        getParserState().getStringPool().releaseString(dataIndex);
    }
    /**
     *
     */
    public void processingInstruction(int targetIndex, int dataIndex) throws Exception {
        getParserState().getStringPool().releaseString(targetIndex);
        getParserState().getStringPool().releaseString(dataIndex);
    }
    /**
     *
     */
    public void comment(int dataIndex) throws Exception {
        getParserState().getStringPool().releaseString(dataIndex);
    }
    /**
     * Not called.
     */
    public void characters(char ch[], int start, int length, boolean cdataSection) throws Exception {
    }
    /**
     * Not called.
     */
    public void ignorableWhitespace(char ch[], int start, int length, boolean cdataSection) throws Exception {
    }
}
