/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.parsers;

import com.ibm.xml.framework.*;
import org.w3c.dom.*;
import org.xml.sax.AttributeList;
import java.util.Stack;

import com.ibm.xml.parser.*;

/**
 *
 * @version Revision: 53 1.21 src/com/ibm/xml/parsers/NonValidatingTXDOMParser.java, parser, xml4j2, xml4j2_0_13 
 */
public class NonValidatingTXDOMParser extends XMLParser
{
    //
    // Constants.
    //
    public static final boolean DEBUG = false;

    //
    // Instance variables.
    //
    protected String fDocumentClass = "com.ibm.xml.parser.TXDocument";
    protected int fRootElementName = -1;
    protected TXDocument fDocument = null;
    protected DTD fDocumentType = null;
    protected com.ibm.xml.parser.EntityPool fEntityPool = null;
    protected Node fCurrentParent = null;
    protected Node fCurrentNode = null;
    protected Stack fNodeStack = new Stack();
    protected boolean fWithinElement = false;
    protected boolean fNormalizeTextNodes = true;
    protected boolean fExpandEntityReferences = false;
    int fAmpIndex = -1;
    int fLtIndex = -1;
    int fGtIndex = -1;
    int fAposIndex = -1;
    int fQuotIndex = -1;
    
     
    
    /**
     *
     */
    public NonValidatingTXDOMParser() {
        setDocumentHandler(this);
        setDocumentTypeHandler(this);
        getParserState().useDefaultStringPool();
        getParserState().useDefaultAttrPool();
        getParserState().useDefaultEntityPool();
        getParserState().useDefaultElementDeclPool();
        init();
    }
    /**
     *
     */
    public void reset() {
        super.reset();
        init();
    }
    /**
     *
     */
    protected void resetOrCopy() {
        super.resetOrCopy();
        init();
    }
    protected void init() {
        fRootElementName = -1;
        fDocument = null;
        fDocumentType = null;
        fCurrentParent = null;
        fCurrentNode = null;
        fNodeStack.removeAllElements();
        fWithinElement = false;
        fNormalizeTextNodes = true;
        fExpandEntityReferences = false;
        com.ibm.xml.framework.StringPool fStringPool = getParserState().getStringPool();
        fAmpIndex = fStringPool.addSymbol("amp");
        fLtIndex = fStringPool.addSymbol("lt");
        fGtIndex = fStringPool.addSymbol("gt");
        fAposIndex = fStringPool.addSymbol("apos");
        fQuotIndex = fStringPool.addSymbol("quot");
    }
    /**
     *
     */
    protected void checkHandlers() throws Exception {
        super.checkHandlers();
        if (fDocument == null)
            fDocument = (TXDocument)(Class.forName(fDocumentClass).newInstance());
    }

    //
    // TX EntityPool compatibility...
    //
    private void initDTDEntityPool() {
        fEntityPool = new com.ibm.xml.parser.EntityPool();
        fEntityPool.add(fDocument.createEntityDecl("lt", "&#60;", false));
        fEntityPool.add(fDocument.createEntityDecl("gt", "&#62;", false));
        fEntityPool.add(fDocument.createEntityDecl("amp", "&#38;", false));
        fEntityPool.add(fDocument.createEntityDecl("apos", "&#39;", false));
        fEntityPool.add(fDocument.createEntityDecl("quot", "&#34;", false));
    }
    
    //
    // Setters and Getters for DOM Parser behaviour modification.
    //
    public void setNormalizeTextNodes(boolean normalize) {
        fNormalizeTextNodes = normalize;
    }

    /**
     * Sets whether <tt>EntityReference</tt> Nodes are expanded (replaced) 
     * by this parser. Expanded means that the children of the 
     * <tt>EntityReference</tt> Node replace it.
     * <p>
     * The default value is false, <tt>EntityReference</tt> Nodes are kept.
     *
     * @param keep  <code>=true</code> means expanding entity references;
     *              <code>=false</code> means maintain entity references.
     *
     * @see org.w3c.dom.EntityReference
     * @see com.ibm.xml.parser.GeneralReference
     */
    public void setExpandEntityReferences(boolean expand) {
        fExpandEntityReferences = expand;
    }
    
    /**
     * Return a boolean which says whether <tt>EntityReference</tt> 
     * Nodes are expanded (replaced) by this parser. Expanded means 
     * that the children of the <tt>EntityReference</tt> Node replace it.
     * <p>
     * The default value is false, <tt>EntityReference</tt> Nodes are kept.
     *
     * @param keep  <code>=true</code> means expanding entity references;
     *              <code>=false</code> means maintain entity references.
     *
     * @see org.w3c.dom.EntityReference
     * @see com.ibm.xml.parser.GeneralReference
     */
    public boolean getExpandEntityReferences() {
        return fExpandEntityReferences;
    }
    
    //
    // XMLDocumentTypeHandler interface
    //

    /**
     *  This function will be called when a &lt;!DOCTYPE...&gt; declaration is
     *  encountered.
     */
    public void doctypeDecl(int rootElementNameIndex) throws Exception {

        if (DEBUG) System.out.println("doctypeDecl(int rootElementNameIndex)="+rootElementNameIndex);

        fRootElementName = rootElementNameIndex;
        com.ibm.xml.framework.StringPool fStringPool = getParserState().getStringPool();
        String docName = fStringPool.toString(fRootElementName);

        fDocumentType = (DTD)fDocument.createDTD(docName, null);

        if (fDocumentType==null) return;

        initDTDEntityPool();
        fDocumentType.setEntityPool(fEntityPool);

        fCurrentParent.appendChild(fDocumentType);
        fCurrentNode = fDocumentType;
    }

    /**
     *  This function will be called at the start of the "internal subset" of a
     *  doctype declaration.
     */
    public void startInternalSubset() throws Exception {

        if (DEBUG) System.out.println("startInternalSubset");

        if (fDocumentType == null) return;

        fDocumentType.setParsingExternal(false);
    }

    /**
     *  This function will be called at the end of the "internal subset" of a
     *  doctype declaration.
     */
    public void endInternalSubset() throws Exception {
        if (DEBUG) System.out.println("endInternalSubset");
        //nothing is done here because parsing the internal subset is the default.
    }

    /**
     *  This function will be called at the start of the "external subset" of a
     *  doctype declaration.
     */
    public void startExternalSubset(int publicIdStringIndex, int systemIdStringIndex) throws Exception {

        if (DEBUG) System.out.println("startExternalSubset(int publicIdStringIndex, int systemIdStringIndex)="+publicIdStringIndex+","+systemIdStringIndex);
        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool fStringPool = getParserState().getStringPool();
        String docName = fStringPool.toString(fRootElementName);


        if (publicIdStringIndex != -1 && systemIdStringIndex != -1) {
            fDocumentType.setExternalID(
                new ExternalID(fStringPool.toString(publicIdStringIndex),
                               fStringPool.toString(systemIdStringIndex)));

        } else
        if (systemIdStringIndex != -1) {
            fDocumentType.setExternalID(
                new ExternalID(fStringPool.toString(systemIdStringIndex)));
        }
        fDocumentType.setParsingExternal(true);
    }

    /**
     *  This function will be called at the end of the "external subset" of a
     *  doctype declaration.
     */
    public void endExternalSubset() throws Exception {

        if (DEBUG) System.out.println("endExternalSubset");
        if (fDocumentType == null) return;

        // called here because parsing the internal subset is the default,
        // AND the "external subset" is usu parsed AFTER the "internal subset",
        // so we want to set the flag back.
        fDocumentType.setParsingExternal(false);
    }


    /**
     *  Build the TX-style CMNodes and ElementDecl and attatch to the fDocumentType (DTD)
     */
    public void elementDecl(int elementIndex) throws Exception {

        if (fDocumentType == null) return;

        ElementDeclPool edp = getParserState().getElementDeclPool();
        String name = getParserState().getStringPool().toString(edp.getElementName(elementIndex));
        int cst = edp.getContentSpecType(elementIndex);
        com.ibm.xml.parser.ContentModel contentModel = null;
        com.ibm.xml.parser.ElementDecl elementDecl = null;

        switch (cst) {
        case com.ibm.xml.framework.ElementDecl.CONTENTSPEC_EMPTY:
            contentModel = fDocument.createContentModel(
                com.ibm.xml.parser.ElementDecl.EMPTY);
            break;
        case com.ibm.xml.framework.ElementDecl.CONTENTSPEC_ANY:
            contentModel = fDocument.createContentModel(
                com.ibm.xml.parser.ElementDecl.ANY);
            break;
        case com.ibm.xml.framework.ElementDecl.CONTENTSPEC_MIXED:
        case com.ibm.xml.framework.ElementDecl.CONTENTSPEC_CHILDREN:
            int contentSpec =  edp.getContentSpec(elementIndex);
            contentModel = fDocument.createContentModel(
                buildCMNode( elementIndex, contentSpec));
            break;
        }
        if (contentModel != null) {
            elementDecl = fDocument.createElementDecl(
                name, contentModel);

            if (elementDecl==null) return;

            fDocumentType.appendChild(elementDecl);
            fCurrentNode = elementDecl;
        }

    }

    /**
     *  This function builds the TX-style CMNodes for an ElementDecl and returns
     *  the top level com.ibm.xml.parser.CMNode.
     */
    protected com.ibm.xml.parser.CMNode buildCMNode(int elementIndex, int contentSpecIndex) {

        com.ibm.xml.framework.StringPool fStringPool = getParserState().getStringPool();
        ElementDeclPool edp = getParserState().getElementDeclPool();
        ContentSpecNode contentSpecNode = new ContentSpecNode();
        ContentSpecNode newSpecNode = new ContentSpecNode();
        edp.getContentSpecNode(contentSpecIndex, contentSpecNode);
        int type = contentSpecNode.type;
        int value = contentSpecNode.value;
        switch (type) {
        case ContentSpecNode.NODE_LEAF:
            String leafString = (value == -1 ? "#PCDATA" : fStringPool.toString(value));
            return new com.ibm.xml.parser.CMLeaf(leafString);
        case ContentSpecNode.NODE_ZERO_OR_ONE:
            edp.getContentSpecNode(contentSpecIndex, newSpecNode);
            return new com.ibm.xml.parser.CM1op('?',buildCMNode(elementIndex, newSpecNode.value));
        case ContentSpecNode.NODE_ZERO_OR_MORE:
            edp.getContentSpecNode(contentSpecIndex, newSpecNode);
            return new com.ibm.xml.parser.CM1op('*',buildCMNode(elementIndex, newSpecNode.value));
        case ContentSpecNode.NODE_ONE_OR_MORE:
            edp.getContentSpecNode(contentSpecIndex, newSpecNode);
            return new com.ibm.xml.parser.CM1op('+',buildCMNode(elementIndex, newSpecNode.value));
        case ContentSpecNode.NODE_CHOICE:
            edp.getContentSpecNode(contentSpecIndex, newSpecNode);
            return new com.ibm.xml.parser.CM2op('|',buildCMNode(elementIndex, newSpecNode.value), buildCMNode(elementIndex, newSpecNode.otherValue));
        case ContentSpecNode.NODE_SEQ:
            edp.getContentSpecNode(contentSpecIndex, newSpecNode);
            return new com.ibm.xml.parser.CM2op(',',buildCMNode(elementIndex, newSpecNode.value), buildCMNode(elementIndex, newSpecNode.otherValue));
        default:
            return null;
        }
    }

    /**
     *  This function builds a TX-style AttList and AttDef and add as child to the DTD.
     */
    public void attlistDecl(int elementIndex, int attDefIndex) throws Exception {

        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool fStringPool = getParserState().getStringPool();
        ElementDeclPool edp = getParserState().getElementDeclPool();
        int elementNameIndex = edp.getElementName( elementIndex);

        String elementName = fStringPool.toString(elementNameIndex);
        com.ibm.xml.parser.Attlist al = fDocument.createAttlist(fStringPool.toString(elementNameIndex));
        com.ibm.xml.parser.AttDef ad = fDocument.createAttDef(fStringPool.toString(edp.getAttName(attDefIndex)));

        if (al==null || ad==null) return;

        if (edp.getAttType(attDefIndex) < com.ibm.xml.framework.AttDef.ENUMERATION) {
            // revisit: The com.ibm.xml.framework enum starts at 0,
            // and the old XML4J1 parser enum starts at 1, with the same ordering.
            // If the ordering changes, then the next statement should be replaced with
            // a switch statement.
            ad.setDeclaredType(edp.getAttType(attDefIndex)+1);
        } else
        if (edp.getAttType(attDefIndex) == com.ibm.xml.framework.AttDef.ENUMERATION
        ) {
                ad.setDeclaredType(com.ibm.xml.parser.AttDef.NAME_TOKEN_GROUP);
        } else {
            // revisit: If the type is UNKNOWN we currently print an error and return.
            System.err.println("ERROR: UNKNOWN AttType:"+edp.getAttType(attDefIndex));
            return;
        }
        if (edp.getAttType(attDefIndex) == com.ibm.xml.framework.AttDef.ENUMERATION
         |  edp.getAttType(attDefIndex) == com.ibm.xml.framework.AttDef.NOTATION
        ) {

            int[] names = fStringPool.stringsInList(edp.getEnumeration(attDefIndex));
            for (int i=0; i < names.length; i++) {
                ad.addElement(fStringPool.toString(names[i]));
            }
        }

        //revisit: These are mappings between the
        // com.ibm.xml.framework.AttDef
        // and
        // com.ibm.xml.parser.AttDef
        switch (edp.getAttDefaultType(attDefIndex)) {
            case com.ibm.xml.framework.AttDef.DEFAULT:
                  ad.setDefaultType(com.ibm.xml.parser.AttDef.NOFIXED);
                  break;
            case com.ibm.xml.framework.AttDef.REQUIRED:
                  ad.setDefaultType(com.ibm.xml.parser.AttDef.REQUIRED);
                  break;
            case com.ibm.xml.framework.AttDef.IMPLIED:
                  ad.setDefaultType(com.ibm.xml.parser.AttDef.IMPLIED);
                  break;
            case com.ibm.xml.framework.AttDef.FIXED:
                  ad.setDefaultType(com.ibm.xml.parser.AttDef.FIXED);
                  break;
            default:
                  ad.setDefaultType(com.ibm.xml.parser.AttDef.UNKNOWN);
                  break;

        }
        int attValueIndex = edp.getAttValue(attDefIndex);
        if (attValueIndex != -1) {
            ad.setDefaultStringValue(fStringPool.toString(attValueIndex));
        }
        al.addElement(ad);

        fDocumentType.appendChild(al);
        fCurrentNode = al;

    } //end attlistDecl


    /**
     *  Append an internal EntityDecl as a Child to the DTD.
     */
    public void internalEntityDecl(int entityIndex) throws Exception {
        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool stringPool = getParserState().getStringPool();
        com.ibm.xml.framework.EntityPool entityPool = getParserState().getEntityPool();
        String name = stringPool.toString(entityPool.getEntityName(entityIndex));
        String value = stringPool.toString(entityPool.getEntityValue(entityIndex));

        com.ibm.xml.parser.EntityDecl entityDecl =
        (com.ibm.xml.parser.EntityDecl)fDocument.createEntityDecl(name, value, false);

        if (entityDecl==null) return;

        Text t = fDocument.createTextNode(value);
        if (t!=null)
            entityDecl.appendChild(t);
        fDocumentType.appendChild(entityDecl);
        fEntityPool.add(entityDecl);
        fCurrentNode = entityDecl;
    }
    /**
     *  Append an external EntityDecl as a Child to the DTD.
     */
    public void externalEntityDecl(int entityIndex) throws Exception {
        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool stringPool = getParserState().getStringPool();
        com.ibm.xml.framework.EntityPool entityPool = getParserState().getEntityPool();
        String name = stringPool.toString(entityPool.getEntityName(entityIndex));
        String publicId = stringPool.toString(entityPool.getPublicId(entityIndex));
        String systemId = stringPool.toString(entityPool.getSystemId(entityIndex));

        com.ibm.xml.parser.EntityDecl entityDecl =
            (com.ibm.xml.parser.EntityDecl)fDocument.createEntityDecl(
                name, new ExternalID(publicId, systemId), false, null);

        if (entityDecl==null) return;
        
        /***
        // Does this make sense here?
        String value = stringPool.toString(entityPool.getEntityValue(entityIndex));
        Text t = fDocument.createTextNode(value);
        if (t!=null)
            entityDecl.appendChild(t);
        /***/

        fDocumentType.appendChild(entityDecl);
        fEntityPool.add(entityDecl);
        fCurrentNode = entityDecl;
    }
    /**
     *  Append an unparsed external EntityDecl as a Child to the DTD.
     */
    public void unparsedEntityDecl(int entityIndex) throws Exception {
        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool stringPool = getParserState().getStringPool();
        com.ibm.xml.framework.EntityPool entityPool = getParserState().getEntityPool();
        String name = stringPool.toString(entityPool.getEntityName(entityIndex));
        String publicId = stringPool.toString(entityPool.getPublicId(entityIndex));
        String systemId = stringPool.toString(entityPool.getSystemId(entityIndex));
        String notationName = stringPool.toString(entityPool.getNotationName(entityIndex));

        com.ibm.xml.parser.EntityDecl entityDecl =
            (com.ibm.xml.parser.EntityDecl)fDocument.createEntityDecl(
                name, new ExternalID(publicId, systemId), false, notationName);

        if (entityDecl==null) return;

        /***
        // Does this make sense here?
        String value = stringPool.toString(entityPool.getEntityValue(entityIndex));
        Text t = fDocument.createTextNode(value);
        if (t!=null)
            entityDecl.appendChild(t);
        /***/
      
        fDocumentType.appendChild(entityDecl);
        fEntityPool.add(entityDecl);
        fCurrentNode = entityDecl;
    }
    /**
     *  Append an TXNotation as a Child to the DTD.
     */
    public void notationDecl(int notationIndex) throws Exception {
        if (fDocumentType == null) return;

        com.ibm.xml.framework.StringPool stringPool = getParserState().getStringPool();
        com.ibm.xml.framework.EntityPool entityPool = getParserState().getEntityPool();
        String name = stringPool.toString(entityPool.getNotationName(notationIndex));
        String publicId = stringPool.toString(entityPool.getPublicId(notationIndex));
        String systemId = stringPool.toString(entityPool.getSystemId(notationIndex));

        TXNotation notation = (TXNotation)fDocument.createNotation(
            name, new ExternalID(publicId, systemId));

        if (notation==null) return;

        fDocumentType.appendChild(notation);
        fCurrentNode = notation;
    }

    //
    // XMLDocumentHandler interface
    //

    /**
     *
     */
    public void setDocumentClass(String documentClass) {
        fDocumentClass = documentClass;
    }
    /**
     *
     */
    public Document getDocument() {
        return fDocument;
    }

    /**
     *
     */
    public void startDocument(int versionIndex,
                              int encodingIndex,
                              int standAloneIndex) throws Exception {

        if (fDocument==null) return;

        fCurrentParent = fDocument;
        fCurrentNode = fDocument;

        com.ibm.xml.framework.StringPool  strPool = getParserState().getStringPool();
        String version = strPool.orphanString(versionIndex);
        String encoding = strPool.orphanString(encodingIndex);
        String standAlone = strPool.orphanString(standAloneIndex);
        if (version != null) fDocument.setVersion(version);
        if (encoding != null) fDocument.setEncoding(encoding);
        if (standAlone != null) fDocument.setStandalone(standAlone);
    }
    /**
     *
     */
    public void endDocument() throws Exception {
    }
    /**
     *
     */
    public void startElement(int elementNameIndex, int attrListIndex) throws Exception {
        String elementName = getParserState().getStringPool().toString(elementNameIndex);
        AttributeList attrList = getParserState().getAttrPool().getAttributeList(attrListIndex);
        Element e = fDocument.createElement(elementName);

        if (e==null) return;

        int attrListLength = attrList.getLength();
        for (int i = 0; i < attrListLength; i++) {
            e.setAttribute(attrList.getName(i), attrList.getValue(i));
            // Register IDs with the DTD during initial parse.
            if (fDocumentType != null && attrList.getType(i).equals("ID")) {
                fDocumentType.registID(e, attrList.getValue(i));
            }
        }
        getParserState().getAttrPool().releaseAttrList(attrListIndex);
        fCurrentParent.appendChild(e);
        fNodeStack.push(fCurrentParent);
        fCurrentParent = e;
        fCurrentNode = e;
        fWithinElement = true;
    }
    /**
     *
     */
    public void endElement(int elementNameIndex) throws Exception {
        fCurrentParent = (Node)fNodeStack.pop();
        fCurrentNode = fCurrentParent;
        fWithinElement = false;
    }
    /**
     *
     */
    public void startEntityReference(int entityIndex) throws Exception {
        
        if (fExpandEntityReferences) return;
        
        com.ibm.xml.framework.EntityPool entityPool = getParserState().getEntityPool();
        int entityNameIndex = entityPool.getEntityName(entityIndex);
        if (entityNameIndex == fAmpIndex
        ||  entityNameIndex == fGtIndex
        ||  entityNameIndex == fLtIndex
        ||  entityNameIndex == fAposIndex
        ||  entityNameIndex == fQuotIndex) return;
        
        com.ibm.xml.framework.StringPool stringPool = getParserState().getStringPool();
        String name = stringPool.toString(entityPool.getEntityName(entityIndex));
        EntityReference er = fDocument.createEntityReference(name);

        if (er==null) return;

        fCurrentParent.appendChild(er);
        fNodeStack.push(fCurrentParent);
        fCurrentParent = er;
        fCurrentNode = er;
    }

    /**
     *
     */
    public void endEntityReference(int entityIndex) throws Exception {
        
        if (fExpandEntityReferences) return;
        
        int entityNameIndex = getParserState().getEntityPool().getEntityName(entityIndex);
        if (entityNameIndex == fAmpIndex
        ||  entityNameIndex == fGtIndex
        ||  entityNameIndex == fLtIndex
        ||  entityNameIndex == fAposIndex
        ||  entityNameIndex == fQuotIndex) return;
        
        fCurrentParent = (Node)fNodeStack.pop();
        fCurrentNode = fCurrentParent;
    }
    /**
     *
     */
    public void characters(int dataIndex, boolean cdataSection) throws Exception {
        Text t = null;
        if (cdataSection) {
            t = fDocument.createCDATASection(getParserState().getStringPool().orphanString(dataIndex));
        } else {
            // if the current node is a TEXT_NODE of an ElementNode and not CDATA
            // then simply append to the current node.
            if (
            fNormalizeTextNodes
            && fWithinElement       
            && fCurrentNode.getNodeType() == Node.TEXT_NODE 
            && fCurrentNode.getParentNode().getNodeType() == Node.ELEMENT_NODE
            ) {
                //System.out.println("Text Node normalize, appending:"+getParserState().getStringPool().toString(dataIndex));
                ((Text)fCurrentNode).appendData(getParserState().getStringPool().orphanString(dataIndex));
                return;
            }
            t = fDocument.createTextNode(getParserState().getStringPool().orphanString(dataIndex));
        }

        if (t==null) return;

        fCurrentParent.appendChild(t);
        fCurrentNode = t;
    }
    /**
     *
     */
    public void ignorableWhitespace(int dataIndex, boolean cdataSection) throws Exception {
        Text t = null;
        if (cdataSection) {
            t = fDocument.createCDATASection(getParserState().getStringPool().orphanString(dataIndex));
        } else {
            t = fDocument.createTextNode(getParserState().getStringPool().orphanString(dataIndex));
        }

        if (t==null) return;

        ((TXText) t).setIsIgnorableWhitespace(true);
        fCurrentParent.appendChild(t);
        fCurrentNode = t;
    }
    /**
     *
     */
    public void processingInstruction(int targetIndex, int dataIndex) throws Exception {
        String target = getParserState().getStringPool().orphanString(targetIndex);
        String data = null;
        if (dataIndex >= 0) {
            data = getParserState().getStringPool().orphanString(dataIndex);
        } else {
            data = "";
        }
        ProcessingInstruction pi = fDocument.createProcessingInstruction(target, data);

        if (pi==null) return;

        fCurrentParent.appendChild(pi);
        fCurrentNode = pi;
    }
    /**
     *
     */
    public void comment(int dataIndex) throws Exception {
        Comment c = fDocument.createComment(getParserState().getStringPool().orphanString(dataIndex));

        if (c==null) return;

        fCurrentParent.appendChild(c);
        fCurrentNode = c;
    }
    /**
     * Not called.
     */
    public void characters(char ch[], int start, int length, boolean cdataSection) throws Exception {
        Text t = null;
        if (cdataSection) {
            t = fDocument.createCDATASection(new String(ch, start, length));
        } else {
            // if the current node is a TEXT_NODE of an ElementNode and not CDATA
            // then simply append to the current node.
            if (
            fNormalizeTextNodes
            && fWithinElement
            && fCurrentNode.getNodeType() == Node.TEXT_NODE 
            && fCurrentNode.getParentNode().getNodeType() == Node.ELEMENT_NODE
            ) {
                //System.out.println("Text Node normalize, appending:"+new String(ch, start, length));
                ((Text)fCurrentNode).appendData(new String(ch, start, length));
                return;
            }
            t = fDocument.createTextNode(new String(ch, start, length));
        }
        

        if (t==null) return;

        fCurrentParent.appendChild(t);
        fCurrentNode = t;
    }
    /**
     * Not called.
     */
    public void ignorableWhitespace(char ch[], int start, int length, boolean cdataSection) throws Exception {
        Text t = null;
        if (cdataSection) {
            t = fDocument.createCDATASection(new String(ch, start, length));
        } else {
            t = fDocument.createTextNode(new String(ch, start, length));
        }

        if (t==null) return;

        ((TXText) t).setIsIgnorableWhitespace(true);
        fCurrentParent.appendChild(t);
        fCurrentNode = t;
    }
}
